# draw graphs

import os
import matplotlib.pyplot as plt
import networkx as nx
import numpy as np
import pandas as pd
import itertools
import numpy as np
import matplotlib.lines as mlines
import matplotlib.transforms as mtransforms
# from statistics import mean
from matplotlib import style
from networkConstruction import *



def readGraph( filename ):
	g = {}
	edges = {}
	total_score = ''
	with open(filename,'r') as f:
		for line in f:
			if line == '\n':
				continue
			elif line.strip().startswith('T'):
				spl = line.split(' ')
				total_score = float(spl[-1])
				break
			spl = line.strip().split(':')
			parent_and_val = spl[0].translate(None,'(),\'').split(' ')
			
			val = float(parent_and_val.pop())
			parent = tuple([parent_and_val[k] for k in xrange(0,len(parent_and_val))])
			##
			parent = tuple(sorted(parent)) #
			##
			children = spl[1].strip().translate(None,'(').split(')')
			children.pop()
			child1 = children[0].translate(None,'\',').split(' ')
			child2 = children[1].translate(None,'\',').split(' ')
			child2 = child2[1:]
			child1tup = tuple([child1[k] for k in xrange(0,len(child1))])
			##
			child1tup = tuple(sorted(child1tup)) #
			##
			child2tup = tuple([child2[k] for k in xrange(0,len(child2))])
			##
			child2tup = tuple(sorted(child2tup)) #
			##
			g[parent] = [child1tup,child2tup]
			edges[parent] = val
	return g,edges, total_score

def readNoiseGraph( filename ):
	g = {}
	with open(filename,'r') as f:
		for line in f:
			spl = line.strip().translate(None,'(),\':').split(' ')
			key = tuple([spl[k] for k in xrange(0,len(spl)-1)])
			g[key] = float(spl[-1])
	return g

def readScores( filename ):
	scores = {}
	with open(filename,'r') as f:
		for line in f:
			spl = line.strip().split(':')
			# m = tuple(spl[0].translate(None,'()\'').split(','))
			m = tuple([s.strip() for s in spl[0].translate(None,'()\'').split(',') if s != ''])
			scores[m] = float(spl[1].strip())
	return scores

def readWtGraph( filename , un_scores ):
	scores = {}
	for node in un_scores:
		k = tuple(sorted(node))
		if k in scores:
			scores[k] = min(scores[k],un_scores[node])
		else:
			scores[k] = un_scores[node]
	g = {}
	edges = {}
	total_score = ''
	nodes = []
	with open(filename,'r') as f:
		for line in f:
			if line == '\n':
				continue
			elif line.strip().startswith('T'):
				spl = line.split(' ')
				total_score = float(spl[-1])
				break
			spl = line.strip().split(':')
			parent_and_val = spl[0].translate(None,'(),\'').split(' ')
			
			val = float(parent_and_val.pop())
			parent = tuple([parent_and_val[k] for k in xrange(0,len(parent_and_val))])
			##
			parent = tuple(sorted(parent)) #
			##
			children = spl[1].strip().translate(None,'(').split(')')
			children.pop()
			child1 = children[0].translate(None,'\',').split(' ')
			child2 = children[1].translate(None,'\',').split(' ')
			child2 = child2[1:]
			child1tup = tuple([child1[k] for k in xrange(0,len(child1))])
			##
			child1tup = tuple(sorted(child1tup)) #
			##
			child2tup = tuple([child2[k] for k in xrange(0,len(child2))])
			##
			child2tup = tuple(sorted(child2tup)) #
			##
			g[parent] = [child1tup,child2tup]
			edges[(parent,child1tup)] = val
			edges[(parent,child2tup)] = val
			nodes.extend([parent,child1tup,child2tup])
	nodes = set(nodes)
	nodes = list(nodes)
	for n in nodes:
		# print [c for c in n]
		as_parent = [e for e in edges if n == e[0]]
		if as_parent == [] and len(n)>1:
			children = [(c,) for c in n]
			if n in g:
				g[n].extend(children)
			else:
				g[n] = children
			e_score = scores[tuple(sorted(n))]
			for c in children:
				edges[(tuple(sorted(n)),c)] = e_score
	
	return g,edges, total_score

def combinePandC( g , e ):
	newg = []
	newe = {}
	for p in g:
		for j in g[p]:
			newg.append((p,j))	
			newe[(p,j)] = e[(tuple(sorted(p)),j)]
		# newg.append((p,g[p][0]))
		# newg.append((p,g[p][1]))
		# newe[(p,g[p][0])] = e[(tuple(sorted(p)),g[p][0])]
		# newe[(p,g[p][1])] = e[(tuple(sorted(p)),g[p][1])]
		# newe[(p,g[p][0])] = e[tuple(sorted(p))]
		# newe[(p,g[p][1])] = e[tuple(sorted(p))]
		
	return newg, newe

def countEdges( graphs ):
	edge_counts = {}
	for g in graphs:
		for e in g:
			if e in edge_counts:
				edge_counts[e] += 1
			else:
				edge_counts[e] = 1
			for c in g[e]:
				if c in edge_counts:
					edge_counts[c] += 1
				else:
					edge_counts[c] = 1
	return edge_counts

def countNodes( graphs ):
	node_counts = {}
	for g in graphs:
		for e in g:
			if e in node_counts:
				node_counts[e] += 1.0
			else:
				node_counts[e] = 1.0
	for n in node_counts:
		node_counts[n] = node_counts[n]/float(len(graphs))
	return node_counts

def countEdgesComb( graphs ):
	edge_portion = {}
	for g in graphs:
		for e in g:
			if e in edge_portion:
				edge_portion[e] += 1.0
			else:
				edge_portion[e] = 1.0
	for e in edge_portion:
		edge_portion[e] = edge_portion[e]/float(len(graphs))
	return edge_portion

def readIndArr( filename ):
	inds = {}
	with open(filename, 'r') as f:
		for line in f:
			spl = line.strip().split(':')
			spl1 = spl[0].strip().translate(None,'(),\':').split(' ')
			chs = spl[1].split(',')
			chs = [tuple(x.translate(None,'(),\'')) for x in chs]
			inds[tuple(spl[0])] = chs
	return inds

def getNetworkResults( net_dir ):
	net_files = os.listdir( net_dir )
	noise_to_files = {}
	for f in net_files:
		if any(s in f for s in ['arr','scores','tmp']): continue
		n_val = f.split('.txt')[0]
		if n_val in noise_to_files:
			noise_to_files[n_val].append(net_dir+f)
		else:
			noise_to_files[n_val] = [net_dir+f]
	return noise_to_files

def getScoreResults( net_dir ):
	net_files = os.listdir( net_dir )
	noise_to_files = {}
	for f in net_files:
		# if any(s in f for s in ['arr','scores','tmp']): continue
		if 'scores' not in f: continue
		n_val = f.split('.')[0]
		if n_val in noise_to_files:
			noise_to_files[n_val].append(net_dir+f)
		else:
			noise_to_files[n_val] = [net_dir+f]
	return noise_to_files

def drawGraph( net_dir , save_dir ):
	nf = getNetworkResults(net_dir)
	scr = getScoreResults(net_dir)

	edge_counts = {}
	node_counts = {}
	ecg = {}
	for v in nf:
		ag = []
		ng = []
		eg = []
		scores = readScores( scr[v+'_scores'][0] )
		for f in nf[v]:
			cg,edges,_ = readWtGraph(f,scores)
			ng.append(cg)
			ncg, nce = combinePandC(cg,edges)
			ag.append(ncg)
			eg.append(nce)
		edge_counts[v] = countEdgesComb(ag)
		node_counts[v] = countNodes(ng)
		ecg[v] = nce

	# for v in node_counts:
	for v in ecg:
		# print v
		# curn = node_counts[v]
		curn = ecg[v]
		cure = edge_counts[v]
		g = nx.Graph()
		g.add_edges_from([(n[0],n[1],{'weight':np.exp(curn[n])}) for n in curn])
		# g.add_edges_from([(n[0],n[1],{'weight':np.exp(1)}) for n in curn])
		edges,weights = zip(*nx.get_edge_attributes(g,'weight').items())

		wts = [w/max(weights) for w in weights]
		new_edges = {}
		new_nodes = []

		for n in g.nodes():
			if len(n) <= 4: # & len(n) > 1:
				for nq in n:
					new_edges[(n,tuple([nq]))] = np.exp(1)

		for e in edges:
			not_in = list(set(e[0])-set(e[1]))
			if len(not_in) == 1:
				to = [t for t in e if not_in[0] in list(t)]
				new_edges[(to[0],tuple(not_in))] = weights[edges.index(e)]
				# new_edges.append((to[0],tuple(not_in)))
				new_nodes.append(tuple(not_in))
				# new_weights.append(1)

		# edges.extend(new_edges)
		# weights.extend(new_weights)
		g.add_edges_from([(n[0],n[1],{'weight':new_edges[n]}) for n in new_edges])
		g.add_nodes_from(new_nodes)

		# print g.edges()

		# edges,weights = zip(*nx.get_edge_attributes(g,'weight').items())
		# print weights
		# sort edges
		nodes_list = list(g.nodes())
		# print nodes
		# nodes.sort(key=len, reverse=True)
		# nodes_list = [n for n in nodes]
		nodes_list.sort(key=len)
		# node_len = sorted([len(k) for k in nodes], reverse=True)
		node_len = sorted([len(k) for k in nodes_list])
		sunl = list(set(node_len))
		# sunl.sort(reverse=True)
		sunl.sort()
		counts = {x:node_len.count(x) for x in node_len}
		y = np.linspace(0,1,len(counts)+2)
		y = y[1:len(y)-1]
		y_pos={p[0]:p[1] for p in zip(sunl,y)}

		#dict for x positions per row indexed by length of nodes
		x_pos= {}
		for l in sunl:
			num_nodes = counts[l]
			x = np.linspace(0,1,num_nodes+2)
			x_pos[l]=list(x[1:len(x)-1])


		#### this works ###
		x_pos_by_node = {}
		for n in nodes_list:
			if len(n) != 1:
				efn = [e for e in edges if n in e and max([len(ei) for ei in e])==len(n)]
				children = []
				#
				for e in efn:
					if e[0] == n:
						if e[1] not in x_pos_by_node:
							x_pos_by_node[e[1]] = x_pos[len(e[1])].pop()
						children.append(e[1])
					if e[1] == n:
						if e[0] not in x_pos_by_node:
							x_pos_by_node[e[0]] = x_pos[len(e[0])].pop()
						children.append(e[0])
				x_pos_by_node[n] = np.mean([x_pos_by_node[c] for c in children])


		pos_by_node = {}
		for n in x_pos_by_node:
			if n not in pos_by_node:
				pos_by_node[n] = np.array([x_pos_by_node[n],y_pos[len(n)]])

		label_pos = {p:np.array([pos_by_node[p][0],pos_by_node[p][1]+.03]) for p in pos_by_node}
		nx.draw(g, pos_by_node, node_color='k',node_size=100,node_shape='o', edgelist=edges, edge_color=tuple(wts), width=4.0, edge_cmap=plt.cm.jet, font_size=10)


		nx.draw_networkx_labels(g, label_pos)

		plt.savefig(save_dir+'/network_assembly_'+str(v)+'.png')
		plt.close()
		g.clear()

def figure4(pairdat, tridat, quaddat, alldat, savedir):
	# setup the options
	node_shape = 'o'
	edge_width = 8.0
	font_size = 64
	title_options = {'fontsize': 64, 'weight':'bold'}
	all_nodes = [tuple(sorted(n)) for n in [('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'),
										('M','NA','NP','PB1','PB2'),
										('HA', 'M', 'NA', 'NS'),
										('NA','NP','PA','PB2'),
										('NP', 'PA', 'PB1', 'PB2'),
										('HA', 'M', 'NA', 'NP'),
										('NS', 'PA', 'PB1', 'PB2'),
										('HA', 'M', 'NS', 'PB1'),
										('M', 'NA', 'NS'),
										('PA', 'PB1', 'PB2'),
										('NP', 'PB1', 'PB2'),
										('NP', 'PA', 'PB2'),
										('NS', 'PA'),
										('NA', 'PB1'),
										('NP', 'PB2'),
										('PB1', 'PB2'),
										('PA', 'PB2')]]

	pos_colors = plt.cm.jet(np.linspace(0,1,len(all_nodes)))
	node_colors = {n:c for n,c in zip(all_nodes, pos_colors)}
	for n in [('HA',), ('M',), ('NA',), ('NP',), ('NS',), ('PA',), ('PB1',), ('PB2',)]:
		node_colors[n] = np.array([0.,0.,0.,0.])

	### start figure ###
	fig_height = 30.
	fig_width = 60.
	plt.figure(figsize=(fig_width,fig_height))

	# panel A: all models

	edges = alldat[0]
	scores = alldat[1]
	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
		for c in edges[p]:
			e_fmt.append((tuple(sorted(p)),tuple(sorted(c))))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
		par = sorted([p for p in node_list if n[0] in p], key = len)[0]
		e_fmt.append((par,n))
		node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
		es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': np.exp(es[e])}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens))

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]

	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('HA', 'M', 'NA', 'NS')] = np.array([1.2*(fig_width/3.), y_pos[1]])
	node_positions[('NP', 'PB1', 'PB2')] = np.array([2.*(fig_width/3.), y_pos[1]])
	node_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#


	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]



	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,1)
	plt.title("a.",title_options, loc='left')
	nx.draw(g,
			node_positions,
			node_color=cur_node_colors,
			node_size=node_sizes,
			node_shape='o',
			edgelist=es.keys(),
			edge_color=tuple(es.values()),
			width=edge_width,
			edge_cmap=plt.cm.viridis,
			font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])
	label_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('M', 'NA', 'NP', 'PB1', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""
	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()

	###########################################################
	# panel b: pairs and triples
	edges = tridat[0]
	scores = tridat[1]
	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
		for c in edges[p]:
			e_fmt.append((tuple(sorted(p)),tuple(sorted(c))))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
		par = sorted([p for p in node_list if n[0] in p], key = len)[0]
		e_fmt.append((par,n))
		node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
		es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': np.exp(es[e])}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens)) + 1.0

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]

	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('HA', 'M', 'NA', 'NS')] = np.array([1.3*(fig_width/3.), y_pos[2]])

	node_positions[('NP', 'PB1', 'PB2')] = np.array([3.6*(fig_width/5.), y_pos[1]])
	node_positions[('M', 'NA', 'NS')] = np.array([2.3*(fig_width/5.), y_pos[1]])

	node_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#

	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]


	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,2)
	plt.title("b.",title_options, loc='left')
	nx.draw(g,
			node_positions,
			node_color=cur_node_colors,
			node_size=node_sizes,
			node_shape='o',
			edgelist=es.keys(),
			edge_color=tuple(es.values()),
			width=edge_width,
			edge_cmap=plt.cm.viridis,
			font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])

	label_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('M', 'NA', 'NP', 'PB1', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""

	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()


	# panel c: pairs only
	edges = pairdat[0]
	scores = pairdat[1]
	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
		for c in edges[p]:
			e_fmt.append((p,c))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
		par = sorted([p for p in node_list if n[0] in p], key = len)[0]
		e_fmt.append((par,n))
		node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
		es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': np.exp(es[e])}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens)) + 1.0

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]

	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	node_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	node_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	node_positions[('NP', 'PB2')] = np.array([3.8*(fig_width/5.), y_pos[1]])

	node_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PA',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#

	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]


	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,3)
	plt.title("c.",title_options, loc='left')
	nx.draw(g,
			node_positions,
			node_color=cur_node_colors,
			node_size=node_sizes,
			node_shape='o',
			edgelist=es.keys(),
			edge_color=tuple(es.values()),
			width=edge_width,
			edge_cmap=plt.cm.viridis,
			font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])

	label_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PA',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('M', 'NA', 'NP', 'PB1', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""

	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()

	# # panel d: hold out model


	# ('HA', 'M', 'NA', 'NS'): [('HA',), ('M', 'NA', 'NS')
	# ('NP', 'PB1', 'PB2'): [('NP',), ('PB1', 'PB2')
	# ('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'): [('PA',), ('NP', 'PB1', 'PB2'), ('HA', 'M', 'NA', 'NS')]
	                   
	                   
	edges = {('HA', 'M', 'NA', 'NS'): [('HA',), ('M', 'NA', 'NS')], 
	        ('NP', 'PB1', 'PB2'): [('NP',), ('PB1', 'PB2')],
	        ('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'): [('PA',), ('NP', 'PB1', 'PB2'), ('HA', 'M', 'NA', 'NS')]}
	scores = alldat[1]

	# scores = {('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'):1, 
	#             ('NP', 'PB1', 'PB2'):.5, 
	#             ('HA', 'M', 'NA', 'NS'):.5,
	#             ('PB1', 'PB2'):.5,
	#             ('M', 'NA', 'NS'):.5,
	#             ('NP', 'PA', 'PB1', 'PB2'):.5
	#             }
	# scores[('HA',)] = 1
	# scores[('M',)] = 1
	# scores[('NA',)] = 1
	# scores[('NS',)] = 1
	# scores[('PA',)] = 1
	# scores[('NP',)] = 1
	# scores[('PB1',)] = 1
	# scores[('PB2',)] = 1

	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
	    for c in edges[p]:
	        e_fmt.append((p,c))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
	    par = sorted([p for p in node_list if n[0] in p], key = len)[0]
	    e_fmt.append((par,n))
	    node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
	    es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': es[e]}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens)) + 1.0

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]

	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('HA', 'M', 'NA', 'NS')] = np.array([1.37*(fig_width/3.), y_pos[3]])

	node_positions[('M', 'NA', 'NS')] = np.array([1.4*(fig_width/3.), y_pos[2]])
	node_positions[('NP', 'PB1', 'PB2')] = np.array([2.1*(fig_width/3.), y_pos[2]])
	node_positions[('PB1', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])


	node_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#

	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]


	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,4)
	plt.title("d.",title_options, loc='left')
	nx.draw(g,
	        node_positions,
	        node_color=cur_node_colors,
	        node_size=node_sizes,
	        node_shape='o',
	        edgelist=es.keys(),
	        edge_color=tuple(es.values()),
	        width=edge_width,
	        edge_cmap=plt.cm.viridis,
	        font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])

	label_positions[('PA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('HA',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('M', 'NA', 'NP', 'PB1', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""

	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()


	# panel e: 

	# HA M NA NS:HA,M NA NS
	# HA M NA NP NS PA PB1 PB2:NP PA PB1 PB2,HA M NA NS
	edges = {('HA','M','NA','NS'): [('HA',),('M','NA','NS')],('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'): [('NP', 'PA', 'PB1', 'PB2'), ('HA', 'M', 'NA', 'NS')]}
	scores = alldat[1]
	# scores = {('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'):1, ('NP', 'PA', 'PB1', 'PB2'):.7, ('HA', 'M', 'NA', 'NS'):.7}
	# scores[('HA',)] = 1
	# scores[('M',)] = 1
	# scores[('NA',)] = 1
	# scores[('NS',)] = 1
	# scores[('PA',)] = 1
	# scores[('NP',)] = 1
	# scores[('PB1',)] = 1
	# scores[('PB2',)] = 1

	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
	    for c in edges[p]:
	        e_fmt.append((p,c))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
	    par = sorted([p for p in node_list if n[0] in p], key = len)[0]
	    e_fmt.append((par,n))
	    node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
	    es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': es[e]}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens)) + 1.0

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]


	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('HA', 'M', 'NA', 'NS')] = np.array([1.*(fig_width/3.), y_pos[2]])
	node_positions[('M','NA','NS')] = np.array([2.*(fig_width/5.), y_pos[1]])

	node_positions[('NP', 'PA', 'PB1', 'PB2')] = np.array([2.*(fig_width/3.), y_pos[2]])

	node_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PA',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#

	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]


	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,5)
	plt.title("e.",title_options, loc='left')
	nx.draw(g,
	        node_positions,
	        node_color=cur_node_colors,
	        node_size=node_sizes,
	        node_shape='o',
	        edgelist=es.keys(),
	        edge_color=tuple(es.values()),
	        width=edge_width,
	        edge_cmap=plt.cm.viridis,
	        font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('HA', 'M', 'NA', 'NS')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])

	label_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PA',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""

	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()




	# panel f: hold out model

	# NA NP PA PB2:NA,NP PA PB2
	# HA M NA NP NS PA PB1 PB2:HA M NS PB1,NA NP PA PB2

	edges = {('NA', 'NP', 'PA', 'PB2'): [('NA',), ('NP', 'PA', 'PB2')], 
	        ('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'): [('HA', 'M', 'NS', 'PB1'), ('NA', 'NP', 'PA', 'PB2')]}
	scores = alldat[1]
	# scores = {('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2'):1, 
	#             ('HA', 'M', 'NS', 'PB1'):.3, 
	#             ('NP', 'PA', 'PB2'):.2,
	#             ('NA', 'NP', 'PA', 'PB2'):.2,
	#             }
	# scores[('HA',)] = 1
	# scores[('M',)] = 1
	# scores[('NA',)] = 1
	# scores[('NS',)] = 1
	# scores[('PA',)] = 1
	# scores[('NP',)] = 1
	# scores[('PB1',)] = 1
	# scores[('PB2',)] = 1

	# get edges in right format
	# list of each individual edges with parent -> child
	# here, we'll fill in the right single children
	e_fmt = []
	for p in edges:
	    for c in edges[p]:
	        e_fmt.append((p,c))

	# get a simple list of all nodes
	from itertools import chain
	node_list = sorted(list(chain.from_iterable(e_fmt)), key=len)

	# get the single children that are not in the edges
	notin = ['HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2']
	notin = [(n,) for n in notin if (n,) not in node_list]

	# find the smallest parent of these singles
	for n in notin:
	    par = sorted([p for p in node_list if n[0] in p], key = len)[0]
	    e_fmt.append((par,n))
	    node_list.append(n)

	node_list = sorted(list(set(node_list)), key = len)

	# get the scores for each one of these eges
	es = {}
	for e in e_fmt:
	    es[e] = scores[e[0]]

	# make the graph
	g = nx.Graph()
	g.add_edges_from([(e[0], e[1], {'weight': es[e]}) for e in es])
	ge,gw = zip(*nx.get_edge_attributes(g,'weight').items())

	# normalize edge weights
	wts = [w/max(gw) for w in gw]

	node_lens = sorted(list(set([len(n) for n in node_list])))
	y_levels = float(len(node_lens)) + 1.0

	# y_positions, leaving buffer
	y_gap = fig_height/y_levels
	y_pos = [i*y_gap for i in range(1,int(y_levels))]
	y_pos[0] = y_pos[0]
	y_pos[1] = y_pos[1]

	# start with top node
	node_positions = {}
	to_plot = []

	# reverse node_list length
	node_list = sorted(node_list, key = len, reverse = True)

	node_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	node_positions[('HA', 'M', 'NS', 'PB1')] = np.array([1.*(fig_width/3.), y_pos[2]])

	node_positions[('NA', 'NP', 'PA', 'PB2')] = np.array([2.*(fig_width/3.), y_pos[2]])
	node_positions[('NP', 'PA', 'PB2')] = np.array([3.1*(fig_width/5.), y_pos[1]])


	node_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('M',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NS',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB1',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NP',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PA',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('PB2',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.])#
	node_positions[('NA',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.])#

	b = 300
	node_sizes = [0 if len(n) == 1 else b*len(n) for n in g.nodes()]


	cur_node_colors = [node_colors[n] for n in g.nodes()]

	plt.subplot(2,3,6)
	plt.title("f.",title_options, loc='left')
	nx.draw(g,
	        node_positions,
	        node_color=cur_node_colors,
	        node_size=node_sizes,
	        node_shape='o',
	        edgelist=es.keys(),
	        edge_color=tuple(es.values()),
	        width=edge_width,
	        edge_cmap=plt.cm.viridis,
	        font_size=font_size)

	label_positions = {}
	label_positions[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = np.array([fig_width/2., y_pos[-1]])
	label_positions[('M', 'NA', 'NP', 'PB1', 'PB2')] = np.array([1.9*(fig_width/3.), y_pos[2]])

	label_positions[('NS', 'PA')] = np.array([1.6*(fig_width/5.), y_pos[1]])
	label_positions[('NA', 'PB1')] = np.array([3.*(fig_width/5.), y_pos[1]])
	label_positions[('NP', 'PB2')] = np.array([4.*(fig_width/5.), y_pos[1]])

	label_positions[('HA',)] = np.array([1.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('M',)] = np.array([2.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NS',)] = np.array([3.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB1',)] = np.array([4.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NP',)] = np.array([5.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PA',)] = np.array([6.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('PB2',)] = np.array([7.*(fig_width/9.), y_pos[0] - 0.0])
	label_positions[('NA',)] = np.array([8.*(fig_width/9.), y_pos[0] - 0.0])

	labels = {}
	labels[('PA',)] = 'PA'
	labels[('HA',)] = 'HA'
	labels[('M',)] = 'M'
	labels[('NS',)] = 'NS'
	labels[('NA',)] = 'NA'
	labels[('NP',)] = 'NP'
	labels[('PB1',)] = 'PB1'
	labels[('PB2',)] = 'PB2'
	labels[('NS', 'PA')] = ""
	labels[('NA', 'PB1')] = ""
	labels[('NP', 'PB2')] = ""
	labels[('M', 'NA', 'NP', 'PB1', 'PB2')] = ""
	labels[('HA', 'M', 'NA', 'NP', 'NS', 'PA', 'PB1', 'PB2')] = ""

	nx.draw_networkx_labels(g, label_positions, labels, font_size=font_size)
	g.clear()
	plt.tight_layout(pad=10.0, w_pad=10.0, h_pad=10.0)
	plt.savefig(savedir+'Figure4.png')
	plt.close()

def figure2( pl , npl , cpl , cnpl , savedir , rnas):
	res_file = savedir+"/pair_likes_circos.txt"
	chr_pos = {}

	p = 0
	for k in rnas:
		chr_pos[k] = (p,p+1)
		p += 2


	str_toprint = []
	for k in pl.keys():
		if k[0] == k[1]:
			continue
		r1pos = chr_pos[k[0]]
		r2pos = chr_pos[k[1]]
		str_toprint.append(','.join(['chr1',str(r1pos[0]),str(r1pos[1]),'chr1',str(r2pos[0]),str(r2pos[1]),k[0],k[1],str(pl[k]),str(npl[k]),str(cpl[k]),str(cnpl[k])])+'\n')

	with open(res_file,'w') as f:
		f.write('Chromosome,chromStart,chromEnd,Chromosome.1,chromStart.1,chromEnd.1,rna1,rna2,pair,npair,cpair,cnpair\n')
		for s in str_toprint:
			f.write(s)

def figure3( pairdat , tridat , quaddat , rnas , pair_scores , tri_scores , savedir ):
	if os.path.exists(savedir+'/Figure_3.png') or os.path.exists(savedir+'/Figure_3a.png') or os.path.exists(savedir+'/Figure_3b.png'):
		pass
	else:
		genKeys_norep = lambda rnas,order: list(itertools.combinations(rnas,order))
		genKeys = lambda rnas,order: list(itertools.permutations(rnas,order))

		pl = pairdat[0]
		npl = pairdat[1]
		cpl = pairdat[2]
		cnpl = pairdat[3]

		tl = tridat[0]
		ntl = tridat[1]
		ctl = tridat[2]
		cntl = tridat[3]

		ql = quaddat[0]
		nql = quaddat[1]
		cql = quaddat[2]
		cnql = quaddat[3]


		# merge tri and quad results (these are observed results, from trained models)
		# merge results dict
		tql = mergeDicts(tl,ntl,ctl,cntl,ql,nql,cql,cnql)

		# get keys and values in some order
		qt_index = sorted(tql.keys(), key=len)
		qt_values = [tql[k] for k in tql]

		# keep only those in observed tri/quad
		pair_values = [pair_scores[tuple(sorted(k))] for k in qt_index]
		tri_values = [tri_scores[tuple(sorted(k))] for k in qt_index]

		# fix row names
		qt_index = [" ".join(k) for k in qt_index]

		qt_df = pd.DataFrame(qt_values, index=qt_index, columns = ["Observed"])
		pair_df = pd.DataFrame(pair_values, index=qt_index, columns = ["Pair calculated"])
		tri_df = pd.DataFrame(tri_values, index=qt_index, columns = ["Pair+Tri calculated"])

		df = pd.concat([qt_df,pair_df,tri_df], axis = 1, join="inner")

		df.to_csv(savedir+"/score_comparison.csv",sep=",")

		row_names = list(df.index)
		tris = [len(x.split(" ")) == 3 for x in row_names]
		trip = df[tris]
		quads = [len(x.split(" ")) == 4 for x in row_names]
		quad = df[quads]


		### plot pair predictions
		fig, ax = plt.subplots()
		ax.scatter(trip[["Observed"]], trip[["Pair calculated"]]/2, color='green', label='triples')
		ax.scatter(quad[["Observed"]], quad[["Pair calculated"]]/2, color='blue', label='quads')

		line = mlines.Line2D([-10, 0], [-10, 0], color='red')
		plt.xlabel("Observed")
		plt.ylabel("Pair calculated")
		ax.add_line(line)
		ax.legend()
		plt.savefig(savedir+'Figure_3a.png')
		plt.close()

		# plot pair + triple predictions
		fig, ax = plt.subplots()
		# ax.scatter(df[["Observed"]], df[["Pair calculated"]], color='green', label='triples')
		ax.scatter(quad[["Observed"]], quad[["Pair+Tri calculated"]]/2, color='blue', label='quads')
		# line = mlines.Line2D([-1, 1], [-1, 1], color='red')


		line = mlines.Line2D([-10, 0], [-10, 0], color='red')
		plt.xlabel("Observed")
		plt.ylabel("Pair calculated")
		ax.add_line(line)
		ax.legend()
		plt.savefig(savedir+'/Figure_3b.png')
		plt.close()

		style.use('ggplot')

		xs = np.array([1,2,3,4,5], dtype=np.float64)
		ys = np.array([5,4,6,5,6], dtype=np.float64)

		def best_fit_slope_and_intercept(xs,ys):
		    m = (((np.mean(xs)*np.mean(ys)) - np.mean(xs*ys)) /
		         ((np.mean(xs)*np.mean(xs)) - np.mean(xs*xs)))
		    b = np.mean(ys) - m*np.mean(xs)
		    return m, b

		def squared_error(ys_orig,ys_line):
		    return sum((ys_line - ys_orig) * (ys_line - ys_orig))

		def coefficient_of_determination(ys_orig,ys_line):
		    y_mean_line = [np.mean(ys_orig) for y in ys_orig]
		    squared_error_regr = squared_error(ys_orig, ys_line)
		    squared_error_y_mean = squared_error(ys_orig, y_mean_line)
		    return 1 - (squared_error_regr/squared_error_y_mean)
		    
		# m, b = best_fit_slope_and_intercept(xs,ys)
		m = 1
		b = 0

		not_in = ["PB2 NA NS","PB2 NS NA","PB2 PB1 NA NS","PB2 NS NA PB1","PB2 NA PB1 NS","PB2 NA NS PB1","PB2 PB1 NS NA","PB2 NS PB1 NA","PB2 PB1 NS","PB2 NS PB1","PB2 PB1 NA","PB2 NA PB1"]

		ys = trip[["Observed"]]
		ys.append(quad[["Observed"]])
		xs = trip[["Pair calculated"]]/2
		xs.append(quad[["Pair calculated"]]/2)


		xs = [xs.values[x] for x in range(0,len(xs.values)) if xs.index[x] not in not_in]
		ys = [ys.values[x] for x in range(0,len(ys.values)) if ys.index[x] not in not_in]

		# xs = [i for i in xs.values]
		xs = [x[0] for x in xs]
		# ys = [i for i in ys.values]
		ys = [x[0] for x in ys]
		xs = np.array(xs)
		ys = np.array(ys)
		regression_line = [(m*x)+b for x in xs]

		r_squared = coefficient_of_determination(ys,regression_line)
		print(r_squared)
		# fig, ax = plt.subplots()
		# ax.scatter(df[["Observed"]], df[["Pair+Tri calculated"]])
		# line = mlines.Line2D([-10, 0], [-10, 0], color='red')
		# ax.add_line(line)
		# ax.xlabel("Observed","pair+tri calculated")
		# plt.savefig('/Users/tmajaria/Documents/projects/flu/score_tris.png')
		# plt.close()
