## Circos for flu segment interaction
library(data.table)
library(RColorBrewer)

# load data
setwd("..")
pwd = getwd()
dat.file = paste(pwd, "results/final_results/pair_likes_circos.txt")
dat.df <- fread(dat.file,sep=",",header=T,stringsAsFactors=FALSE,showProgress=TRUE,data.table=FALSE)

# convert NAs to strings
for (v in seq(1,length(dat.df$rna1))){
  if (is.na(dat.df$rna1[v])){
    dat.df$rna1[v] <- "NA"
  }
  if (is.na(dat.df$rna2[v])){
    dat.df$rna2[v] <- "NA"
  }
  if (dat.df$npair[v] == 0){
    dat.df$npair[v] = -20
  }
  if (dat.df$cpair[v] == 0){
    dat.df$cpair[v] = -20
  }
  if (dat.df$cnpair[v] == 0){
    dat.df$cnpair[v] = -20
  }
}

# remove weight columns
dat.pos <- dat.df[,1:6]
names(dat.pos) <- c("chromA","chromStartA", "chromEndA", "chromB", "chromStartB", "chromEndB")

dat.chr <- data.frame(id=c('ia1','ia2','ia3','ia4','ia5','ia6','ia7','ia8'), name=c('PB2','PB1','PA','HA','NP','NA','M','NS'), start=rep(0,8), end = c(2341,2341,2233,1778,1565,1413,1027,890))
dat.link <- dat.df[,c("rna1","rna2","cpair","npair","cnpair","pair")]

dat.link <- dat.link[dat.link$npair >= -.23,]
dat.link$pair <- dat.link$npair

final.links <- list()
colors <- colorRampPalette(brewer.pal(11,"Spectral"))(NROW(dat.link))
for (row in seq(1,NROW(dat.link))){
  rna1id <- as.character(dat.chr$id[which(dat.chr$name == dat.link$rna1[row])])
  rna2id <- as.character(dat.chr$id[which(dat.chr$name == dat.link$rna2[row])])
  # outsize_half <- ceiling((dat.chr$end[which(dat.chr$id==rna1id)])*exp(dat.df$pair[row])**2/2)
  # outsize_half <- ceiling((1000)*exp(dat.df$pair[row])**2/2)
  outsize_half <- ceiling((1000)*exp(dat.df$pair[row])**(1/2))
  middle <- ceiling((dat.chr$end[which(dat.chr$id==rna1id)])/2)
  left1 <- middle-outsize_half
  right1 <- middle+outsize_half
  middle <- ceiling((dat.chr$end[which(dat.chr$id==rna2id)])/2)
  left2 <- middle-1
  right2 <- middle+1
  color <- col2rgb(colors[row])
  final.links[[row]] <- data.frame(id1=rna1id,start1=left1,end1=right1,id2=rna2id,start2=left2,end2=right2,color=paste("color=(",color[1],",",color[2],",",color[3],",0.3",")",sep=""))
  
}

links <- do.call(rbind,final.links)
fwrite(links, file= paste(pwd,"/code/circos-0.69-6/flu.links.txt", sep=" ", row.names = F,col.names = F)
