import os

def getFilepaths():
	base_dir = '/'.join(os.getcwd().split('/')[:-1])
	data_dir = base_dir+'/'+'data/'
	results_dir = base_dir+'/'+'results/'
	intermediate_results_dir = results_dir+'intermediate_results/'
	final_results_dir = results_dir+'final_results/'
	image_dir = base_dir+'/images/'
	script_dir = base_dir+'/scripts/'
	table_figure_dir = results_dir+'/fig_tab/'

	image_data_dir = intermediate_results_dir+'/imgdat/'

	# check that we have all the directories
	if not os.path.exists(data_dir):
		print 'Data directory does not exist, exiting'
		os.sys.exit(1)
	if not os.path.exists(results_dir):
		os.mkdir(results_dir)
	if not os.path.exists(intermediate_results_dir):
		os.mkdir(intermediate_results_dir)
	if not os.path.exists(final_results_dir):
		os.mkdir(final_results_dir)
	if not os.path.exists(image_data_dir):
		print 'Preprocessed image data directory does not exist, exiting'
		os.sys.exit(1)
	if not os.path.exists(table_figure_dir):
		os.mkdir(table_figure_dir)


	# read in probe file
	probe_file = base_dir+'/code/prob_key.txt'	
	with open(probe_file,'r') as f:
		probe_lines = f.read().splitlines()

	# get the list of rnas
	probe_key = dict()
	rnas = []
	for line in probe_lines:
		spl = line.split(' ')
		probe_key[spl[0]] = [val if val != 'N' else 'NA' for val in spl[1:] if val!='_' ]
		rnas.extend(set(probe_key[spl[0]]))

	rnas = set(rnas)

	# make a map between rnas and probes
	probes_per_rna = dict()
	for rna in rnas:
		probes_per_rna[rna] = [probe for probe,tags in probe_key.items() if rna in tags]

	# get a dictionary of intermediate results files to rnas
	# these are files that have point positions
	rna_files = dict()
	for rna in probes_per_rna.keys():
		rna_files[rna] = [f for f in os.listdir(image_data_dir) if any(f.startswith(probe.lower()) for probe in probes_per_rna[rna]) and f.endswith('.txt') and rna in f]

	# get a dictionary of cell and nuclear position files
	cell_files = dict()
	nuc_files = dict()
	for probe in probe_key.keys():
		cell_files[probe] = [f for f in os.listdir(image_data_dir) if f.startswith(probe.lower()) and f.endswith('.txt') and 'cell' in f]
		nuc_files[probe] = [f for f in os.listdir(image_data_dir) if f.startswith(probe.lower()) and f.endswith('.txt') and 'nuc' in f]

	# return all data
	return base_dir, data_dir, intermediate_results_dir, final_results_dir, table_figure_dir, rnas, probes_per_rna, rna_files, cell_files, nuc_files


# def getResFilepaths():
# 	# base_dir, dat_dir, rna_idents, probes_per_rna, rna_files, cell_files, nuc_files, box_files = getFilepaths()
# 	base_dir, dat_dir, inter_res_dir, final_res_dir, rna_idents, probes_per_rna, rna_files, cell_files, nuc_files, box_files = getFilepaths()
# 	# res_dir = base_dir+'/results/'
# 	# cdir = res_dir+'inhomog/cell/'
# 	# ndir = res_dir+'inhomog/nuc/'
# 	# # cndir = res_dir+'inhomog/cell_nuc/'
# 	# cndir = res_dir+'inhomog_old/cell_nuc/'
# 	# # pairdir = res_dir+'inhomog/pairs/'
# 	# pairdir = res_dir+'inhomog_old/pairs/'
# 	# cnpairdir = res_dir+'inhomog/cn_pairs/'
# 	# cntridir = res_dir+'inhomog/cn_tri/'
# 	# cnquaddir = res_dir+'inhomog/cn_quad/'
# 	# savedir = base_dir+'fig_tab/'

# 	res_dir = final_res_dir
# 	cdir = res_dir+'inhomog/cell/'
# 	ndir = res_dir+'inhomog/nuc/'
# 	cndir = res_dir+'inhomog/cell_nuc/'
# 	pairdir = res_dir+'inhomog/pairs/'
# 	cnpairdir = res_dir+'inhomog/cn_pairs/'
# 	cntridir = res_dir+'inhomog/cn_tri/'
# 	tridir = res_dir+'inhomog/tri/'
# 	cnquaddir = res_dir+'inhomog/cn_quad/'
# 	quaddir = res_dir+'inhomog/quad/'
# 	savedir = base_dir+'fig_tab/'
# 	pdir = res_dir+'p_vals/'

# 	return res_dir, cdir, ndir, cndir, pairdir, cnpairdir, tridir, cntridir, quaddir, cnquaddir, savedir, pdir

