## masterscript
from getFilepaths import *
from modelTraining import *
from writeTables import *
from networkConstruction import *
from drawGraphs import *
from writeAllNetworks import *
# from figure4 import *

import numpy as np

# get the filepaths that well need to run the rest of the code
base_dir, data_dir, intermediate_results_dir, final_results_dir, table_figure_dir, rnas, probes_per_rna, rna_files, cell_files, nuc_files = getFilepaths()

## make sure that we have directories for each model type
models = [
	'cell',
	'nuc',
	'cn',
	'pair',
	'n_pair',
	'c_pair',
	'cn_pair',
	'tri',
	'n_tri',
	'c_tri',
	'cn_tri',
	'quad',
	'n_quad',
	'c_quad',
	'cn_quad']

model_results_dirs = [final_results_dir+'/'+m+'/' for m in models]

image_data_dir = intermediate_results_dir+'/'+'imgdat/'

if not os.path.isdir(image_data_dir):
	os.mkdir(image_data_dir)

for m in models:
	if not os.path.isdir(final_results_dir+'/'+m):
		os.mkdir(final_results_dir+'/'+m)

## train all of the models
inHomo_cell(base_dir, image_data_dir, model_results_dirs[0], rnas, rna_files)

inHomo_nuc(base_dir, image_data_dir, model_results_dirs[1], rnas, rna_files)

inHomo_cn(base_dir, image_data_dir, model_results_dirs[2], rnas, rna_files)

inHomo_pat(base_dir, image_data_dir, model_results_dirs[3], rnas, rna_files, probes_per_rna)

inHomo_n_pat(base_dir, image_data_dir, model_results_dirs[4], rnas, rna_files, probes_per_rna)

inHomo_c_pat(base_dir, image_data_dir, model_results_dirs[5], rnas, rna_files, probes_per_rna)

inHomo_cn_pat(base_dir, image_data_dir, model_results_dirs[6], rnas, rna_files, probes_per_rna)

inHomo_tri(base_dir, image_data_dir, model_results_dirs[7], rnas, rna_files, probes_per_rna)

inHomo_n_tri(base_dir, image_data_dir, model_results_dirs[8], rnas, rna_files, probes_per_rna)

inHomo_c_tri(base_dir, image_data_dir, model_results_dirs[9], rnas, rna_files, probes_per_rna)

inHomo_cn_tri(base_dir, image_data_dir, model_results_dirs[10], rnas, rna_files, probes_per_rna)

inHomo_quad(base_dir, image_data_dir, model_results_dirs[11], rnas, rna_files, probes_per_rna)

inHomo_n_quad(base_dir, image_data_dir, model_results_dirs[12], rnas, rna_files, probes_per_rna)

inHomo_c_quad(base_dir, image_data_dir, model_results_dirs[13], rnas, rna_files, probes_per_rna)

inHomo_cn_quad(base_dir, image_data_dir, model_results_dirs[14], rnas, rna_files, probes_per_rna)

## run the network assembly
# load the results
cm,cv = getCovarModelRes(model_results_dirs[0],rnas)
nl,nv = getCovarModelRes(model_results_dirs[1],rnas)
cnl, cnv = getCovarModelRes(model_results_dirs[2],rnas)
pl,pv = getPairModelRes(model_results_dirs[3],rnas)
npl,npv = getPairModelRes(model_results_dirs[4],rnas)
cpl,cpv = getPairModelRes(model_results_dirs[5],rnas)
cnpl,cnpv = getPairModelRes(model_results_dirs[6],rnas)
tl, tv = getModelResultsAll(rnas,3,model_results_dirs[7])
ntl, ntv = getModelResultsAll(rnas,3,model_results_dirs[8])
ctl, ctv = getModelResultsAll(rnas,3,model_results_dirs[9])
cntl, cntv = getModelResultsAll(rnas,3,model_results_dirs[10])
ql, qv = getModelResultsAll(rnas,4,model_results_dirs[11])
nql, nqv = getModelResultsAll(rnas,4,model_results_dirs[12])
cql, cqv = getModelResultsAll(rnas,4,model_results_dirs[13])
cnql, cnqv = getModelResultsAll(rnas,4,model_results_dirs[14])

all_likes_q = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl,ql,nql,cql,cnql)
all_likes_p = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl)
all_likes_t = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl)
q_p_only = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,ql,nql,cql,cnql)

# contruct the networks
print 'Pairs only'
scores_p,ind_arr_p, tree_edges_p, fs_p, pkeys = solveTree(rnas,all_likes_p)
print 'Pairs+Triples'
scores_t,ind_arr_t, tree_edges_t, fs_t, tkeys = solveTree(rnas,all_likes_t)
print 'Pairs+Triples+Quads'
scores_q,ind_arr_q, tree_edges_q, fs_q, qkeys = solveTree(rnas,all_likes_q)
print 'Pairs+Quads'
scores_qp,ind_arr_qp, tree_edges_qp, fs_qp, qpkeys = solveTree(rnas,q_p_only)

# # save the results
network_results_dir = final_results_dir+'/network_assembly/'
if not os.path.isdir(network_results_dir):
	os.mkdir(network_results_dir)

if os.path.exists(network_results_dir+'/network_pairs.txt'):
	pass
else:
	saveTree(network_results_dir+'/network_pairs.txt',tree_edges_p,scores_p,fs_p)
	saveScores(network_results_dir+'/network_pairs_scores.txt',pkeys,scores_p)
	saveIndArr(network_results_dir+'/network_pairs_ind_arr.txt',ind_arr_p,pkeys)

if os.path.exists(network_results_dir+'/network_tri.txt'):
	pass
else:
	saveTree(network_results_dir+'/network_tri.txt',tree_edges_t,scores_t,fs_t)
	saveScores(network_results_dir+'/network_tri_scores.txt',tkeys,scores_t)
	saveIndArr(network_results_dir+'/network_tri_ind_arr.txt',ind_arr_t,tkeys)

if os.path.exists(network_results_dir+'/network_quad.txt'):
	pass
else:
	saveTree(network_results_dir+'/network_quad.txt',tree_edges_q,scores_q,fs_q)
	saveScores(network_results_dir+'/network_quad_scores.txt',qkeys,scores_q)
	saveIndArr(network_results_dir+'/network_quad_ind_arr.txt',ind_arr_q,qkeys)

if os.path.exists(network_results_dir+'/network_quad_pairs.txt'):
	pass
else:
	saveTree(network_results_dir+'/network_quad_pair.txt',tree_edges_qp,scores_qp,fs_qp)
	saveScores(network_results_dir+'/network_quad_pair_scores.txt',qpkeys,scores_qp)
	saveIndArr(network_results_dir+'/network_quad_pair_arr.txt',ind_arr_qp,qpkeys)

# ## run noise assembly
noise_results_dirs = final_results_dir+'/noise_network_assembly/'

# if not os.path.isdir(noise_results_dirs):
# 	os.mkdir(noise_results_dirs)

# sd_vals = np.linspace(0.01,2.0,num=5)

# for sd_ind in range(0,len(sd_vals)):
# 	sd = sd_vals[sd_ind]
# 	for i in xrange(1,21):
# 		print 'SD: '+str(sd)+' Run #: '+str(i)
# 		save_file = noise_results_dirs+'/noise_'+str(sd)+'_tree_'+str(i)+'.txt'
# 		ind_file = noise_results_dirs+'/noise_'+str(sd)+'_indarr_'+str(i)+'.txt'
# 		score_file = noise_results_dirs+'/noise_'+str(sd)+'_scores_'+str(i)+'.txt'
# 		tmpfile = noise_results_dirs+'/noise_'+str(sd)+'_tree_'+str(i)+'.tmp'

# 		if os.path.exists(tmpfile) or os.path.exists(save_file):
# 			continue
# 		else:
# 			out = open(tmpfile, 'w')
# 			out.write("lol")
# 			out.close()

# 		scores_q_noise,ind_arr_q_noise, tree_edges_q_noise, fs_q_noise, qkeys_noise = solveTree(rnas,all_likes_q,sd)
# 		saveTree(save_file,tree_edges_q_noise,scores_q_noise,fs_q_noise)
# 		saveIndArr(ind_file, ind_arr_q_noise, qkeys_noise)
# 		saveScores(score_file, qkeys_noise, scores_q_noise)

# 		if os.path.exists(tmpfile):
# 			os.remove(tmpfile)

# # run hold out assembly
# genKeys = lambda rnas,order: list(itertools.permutations(rnas,order))

hold_results_dirs = final_results_dir+'/holdout_network_assembly/'
# if not os.path.isdir(hold_results_dirs):
# 	os.mkdir(hold_results_dirs)

# nodes_to_delete = [['HA','M','NA','NS'], ['NP','PB1','PB2']]
# for node in nodes_to_delete:
# 	k = '_'.join(node)
# 	print k
	
# 	all_likes = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl,ql,nql,cql,cnql)
	
# 	to_del = genKeys(node,len(node))
# 	for k in to_del:
# 		if k in all_likes:
# 			del all_likes[k]
	
# 	k = '_'.join(node)
# 	save_file = hold_results_dirs+'/'+k+'_tree_.txt'
# 	ind_file = hold_results_dirs+'/'+k+'_indarr_.txt'
# 	score_file = hold_results_dirs+'/'+k+'_scores_.txt'
# 	tmpfile = hold_results_dirs+'/'+k+'_tree_.tmp'

# 	if os.path.exists(tmpfile) or os.path.exists(save_file):
# 		continue
# 	else:
# 		out = open(tmpfile, 'w')
# 		out.write("lol")
# 		out.close()

# 	scores, ind_arr, tree_edges, fs, keys = solveTree(rnas,all_likes)

# 	saveTree(save_file,tree_edges,scores,fs)
# 	saveScores(score_file,keys,scores)
# 	saveIndArr(ind_file,ind_arr,keys)
# 	if os.path.exists(tmpfile):
# 		os.remove(tmpfile)

# print "Both"

# all_likes = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl,ql,nql,cql,cnql)

# to_del = genKeys(['HA','M','NA','NS'],4)
# to_del2 = genKeys(['NP','PB1','PB2'],3)
# for k in to_del:
# 	if k in all_likes:
# 		del all_likes[k]
# for k in to_del2:
# 	if k in all_likes:
# 		del all_likes[k]

# k = "both"

# save_file = hold_results_dirs+'/'+k+'_tree_.txt'
# ind_file = hold_results_dirs+'/'+k+'_indarr_.txt'
# score_file = hold_results_dirs+'/'+k+'_scores_.txt'
# tmpfile = hold_results_dirs+'/'+k+'_tree_.tmp'

# scores, ind_arr, tree_edges, fs, keys = solveTree(rnas,all_likes)

# saveTree(save_file,tree_edges,scores,fs)
# saveScores(score_file,keys,scores)
# saveIndArr(ind_file,ind_arr,keys)
# if os.path.exists(tmpfile):
# 	os.remove(tmpfile)

# # run noisy holdout assembly
# sd_vals = np.linspace(0.01,2.0,num=5)

# for sd_ind in xrange(0,len(sd_vals)):
# 	sd = sd_vals[sd_ind]

# 	nodes_to_delete = [['HA','M','NA','NS'], ['NP','PB1','PB2']]
# 	for node in nodes_to_delete:
# 		k = '_'.join(node)
# 		print k
		
# 		all_likes = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl,ql,nql,cql,cnql)
		
# 		to_del = genKeys(node,len(node))
# 		for k in to_del:
# 			if k in all_likes:
# 				del all_likes[k]
		
# 		k = '_'.join(node)
# 		save_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_tree_.txt'
# 		ind_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_indarr_.txt'
# 		score_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_scores_.txt'
# 		tmpfile = hold_results_dirs+'/'+k+'_'+str(sd)+'_tree_.tmp'

# 		scores, ind_arr, tree_edges, fs, keys = solveTree(rnas,all_likes,sd)

# 		saveTree(save_file,tree_edges,scores,fs)
# 		saveScores(score_file,keys,scores)
# 		saveIndArr(ind_file,ind_arr,keys)
# 		if os.path.exists(tmpfile):
# 			os.remove(tmpfile)
# #### need to fix this
# 	print "Both"
# 	for i in xrange(0,20):
# 		print i 
# 		all_likes = mergeDicts(cm,nl,cnl,pl,npl,cpl,cnpl,tl,ntl,ctl,cntl,ql,nql,cql,cnql)
		
# 		to_del = genKeys(['HA','M','NA','NS'],4)
# 		to_del2 = genKeys(['NP','PB1','PB2'],3)
# 		for k in to_del:
# 			if k in all_likes:
# 				del all_likes[k]
# 		for k in to_del2:
# 			if k in all_likes:
# 				del all_likes[k]

# 		k = "both"

# 		save_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_tree_'+str(i)+'.txt'
# 		ind_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_indarr_'+str(i)+'.txt'
# 		score_file = hold_results_dirs+'/'+k+'_'+str(sd)+'_scores_'+str(i)+'.txt'
# 		tmpfile = hold_results_dirs+'/'+k+'_'+str(sd)+'_tree_'+str(i)+'.tmp'
# 		if os.path.exists(tmpfile) or os.path.exists(save_file):
# 				continue
# 		else:
# 			out = open(tmpfile, 'w')
# 			out.write("lol")
# 			out.close()

# 		scores, ind_arr, tree_edges, fs, keys = solveTree(rnas,all_likes,sd)

# 		saveTree(save_file,tree_edges,scores,fs)
# 		saveScores(score_file,keys,scores)
# 		saveIndArr(ind_file,ind_arr,keys)
# 		if os.path.exists(tmpfile):
# 			os.remove(tmpfile)

# write all of the networks to a file
writeModelGraphMapping( noise_results_dirs , hold_results_dirs , final_results_dir )

## make figure 1
# this figure is generated by hand

## make figure 2
figure2( pl , npl , cpl , cnpl , final_results_dir , rnas)

## make figure 3
figure3( [pl,npl,cpl,cnpl] , [tl,ntl,ctl,cntl] , [ql,nql,ctl,cnql] , rnas , scores_p , scores_t , table_figure_dir )

## make figure 4
figure4([tree_edges_p, scores_p], [tree_edges_t, scores_t], [tree_edges_qp, scores_qp], [tree_edges_q, scores_q], table_figure_dir)

## write the tables
runWriteTables( [cm,cv] , [nl,nv] , [cnl,cnv] , [pl,pv] , [npl,npv], [ntl,ntv] , [nql,nqv] , table_figure_dir , final_results_dir , rnas)
