## model training functions
from spp import *

## inhomogenous, dependent on cell distance
def inHomo_cell(base_dir, dat_dir, res_dir, rna_idents, rna_files):
	cov_funcs = [cellDist]
	for rna in rna_idents:
		tmp_file = res_dir+'/'+rna+'.tmp'
		res_file = res_dir+'/'+rna+'.txt'
		beta_file = res_dir+'/'+rna+'_params.txt'

		if os.path.exists(tmp_file) or os.path.exists(res_file):
			continue
		else:
			out = open(tmp_file, 'w')
			out.write("lol")
			out.close()

			out = open(res_file, 'w')
			out.close()

		
		quads = []
		cdists = []
		for ptfile in rna_files[rna]:
			img_pref = ptfile.split('prot')[0]
			pts = np.loadtxt(dat_dir+ptfile, dtype=float)
			cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
			nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)
			cur_pat = PointPattern(pts,mark=rna,cellpat=cell_pts,nucpat=nuc_pts,dimension=3)

			quads.append(QuadScheme(cur_pat,50))
			cdists.append(cellDist(quads[-1]))
			
		likes = []
		betas = []
		failed = []
		for qind in xrange(0,len(quads)):
			q = quads[qind]
			try:
				beta, loglike = FitModel( q , covar=cdists[qind])
				likes.append(loglike)
				betas.append(beta)
			except:
				failed.append(qind)
				pass


		fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

		with open(beta_file,'w') as f:
			for val in betas:
				for b in val:
					f.write(str(b)+' ')
				f.write('\n')

		cv_ll = []
		for qind in xrange(0,len(fit_quads)):
			held = fit_quads[qind]
			mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
			logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
			# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
			cv_ll.append(logl)


			with open(res_file,'a') as f:
				f.write(str(logl/10.)+'\n')

		avg_ll = np.mean(cv_ll)
		var_ll = np.var(cv_ll)
		with open(res_file,'a') as f:
			f.write('\n')
			f.write('mean: '+str(avg_ll)+'\n')
			f.write('var: '+str(var_ll)+'\n')

		# os.remove(tmp_file)

## inhomogenous, dependent on nuclear distance
def inHomo_nuc(base_dir, dat_dir, res_dir, rna_idents, rna_files):
	cov_funcs = [nucDist]
	for rna in rna_idents:
		tmp_file = res_dir+'/'+rna+'.tmp'
		res_file = res_dir+'/'+rna+'.txt'
		beta_file = res_dir+'/'+rna+'_params.txt'

		if os.path.exists(tmp_file) or os.path.exists(res_file):
			continue
		else:
			out = open(tmp_file, 'w')
			out.write("lol")
			out.close()

			out = open(res_file, 'w')
			out.close()

		
		quads = []
		ndists = []
		for ptfile in rna_files[rna]:
			img_pref = ptfile.split('prot')[0]
			pts = np.loadtxt(dat_dir+ptfile, dtype=float)
			cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
			nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

			maxx = max(cell_pts[:,0])
			maxy = max(cell_pts[:,1])
			maxz = max(cell_pts[:,2])

			minx = min(cell_pts[:,0])
			miny = min(cell_pts[:,1])
			minz = min(cell_pts[:,2])

			max_xyz = max(maxx,maxy,maxz)
			min_xyz = min(minx,miny,minz)

			maxx = max_xyz
			maxy = max_xyz
			maxz = max_xyz
			minx = min_xyz
			miny = min_xyz
			minz = min_xyz

			pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
			cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
			nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
			
			cur_pat = PointPattern(pts_scale,mark=rna,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3)

			

			quads.append(QuadScheme(cur_pat,50))
			ndists.append(nucDist(quads[-1]))
			
		likes = []
		betas = []
		failed = []
		for qind in xrange(0,len(quads)):
			q = quads[qind]
			try:
				beta, loglike = FitModel( q , covar=ndists[qind])
				likes.append(loglike)
				betas.append(beta)
			except:
				failed.append(qind)
				pass


		fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

		with open(beta_file,'w') as f:
			for val in betas:
				for b in val:
					f.write(str(b)+' ')
				f.write('\n')

		cv_ll = []
		for qind in xrange(0,len(fit_quads)):
			held = fit_quads[qind]
			mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
			logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
			# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],ndists[qind][held.ptlabs==1]),mean_beta)
			cv_ll.append(logl)


			with open(res_file,'a') as f:
				f.write(str(logl/10.)+'\n')

		avg_ll = np.mean(cv_ll)
		var_ll = np.var(cv_ll)
		with open(res_file,'a') as f:
			f.write('\n')
			f.write('mean: '+str(avg_ll)+'\n')
			f.write('var: '+str(var_ll)+'\n')

		# os.remove(tmp_file)

## inhomogenous, dependent on cell/nuclear ratio
def inHomo_cn(base_dir, dat_dir, res_dir, rna_idents, rna_files):
	cov_funcs = [fracDist]
	for rna in rna_idents:
		tmp_file = res_dir+'/'+rna+'.tmp'
		res_file = res_dir+'/'+rna+'.txt'
		beta_file = res_dir+'/'+rna+'_params.txt'

		if os.path.exists(tmp_file) or os.path.exists(res_file):
			continue
		else:
			out = open(tmp_file, 'w')
			out.write("lol")
			out.close()

			out = open(res_file, 'w')
			out.close()

		
		quads = []
		fdists = []
		for ptfile in rna_files[rna]:
			img_pref = ptfile.split('prot')[0]
			pts = np.loadtxt(dat_dir+ptfile, dtype=float)
			cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
			nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

			maxx = max(cell_pts[:,0])
			maxy = max(cell_pts[:,1])
			maxz = max(cell_pts[:,2])

			minx = min(cell_pts[:,0])
			miny = min(cell_pts[:,1])
			minz = min(cell_pts[:,2])

			max_xyz = max(maxx,maxy,maxz)
			min_xyz = min(minx,miny,minz)

			maxx = max_xyz
			maxy = max_xyz
			maxz = max_xyz
			minx = min_xyz
			miny = min_xyz
			minz = min_xyz

			pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
			cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
			nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

			cur_pat = PointPattern(pts_scale,mark=rna,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3)

			quads.append(QuadScheme(cur_pat,50))
			fdists.append(fracDist(quads[-1]))
			
			
		likes = []
		betas = []
		failed = []
		for qind in xrange(0,len(quads)):
			q = quads[qind]
			try:
				beta, loglike = FitModel( q , covar=fdists[qind])
				likes.append(loglike)
				betas.append(beta)
			except:
				failed.append(qind)
				pass


		fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

		with open(beta_file,'w') as f:
			for val in betas:
				for b in val:
					f.write(str(b)+' ')
				f.write('\n')

		cv_ll = []
		for qind in xrange(0,len(fit_quads)):
			held = fit_quads[qind]
			mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
			logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
			# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],ndists[qind][held.ptlabs==1]),mean_beta)
			cv_ll.append(logl)


			with open(res_file,'a') as f:
				f.write(str(logl/10.)+'\n')

		avg_ll = np.mean(cv_ll)
		var_ll = np.var(cv_ll)
		with open(res_file,'a') as f:
			f.write('\n')
			f.write('mean: '+str(avg_ll)+'\n')
			f.write('var: '+str(var_ll)+'\n')

		# os.remove(tmp_file)

## inhomogenous, dependent on distance to one other pattern
def inHomo_pat(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	cov_funcs = [patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:

			tmp_file = res_dir+'/'+rna1+'_'+rna2+'.tmp'
			res_file = res_dir+'/'+rna1+'_'+rna2+'.txt'
			beta_file = res_dir+'/'+rna1+'_'+rna2+'_params.txt'

			if os.path.exists(tmp_file) or os.path.exists(res_file):
				continue
			else:
				out = open(tmp_file, 'w')
				out.write("lol")
				out.close()

				out = open(res_file, 'w')
				out.close()

			probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2]]
			rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
			rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
			
			
			quads = []
			patdists = []
			for pfind in xrange(0,len(rna1_files)):
				ptfile = rna1_files[pfind]
				img_pref = ptfile.split('prot')[0]
				optfile = img_pref+'prot'+rna2+'.txt'

				pts = np.loadtxt(dat_dir+ptfile, dtype=float)
				opts = np.loadtxt(dat_dir+optfile, dtype=float)
				
				cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
				nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

				maxx = max(cell_pts[:,0])
				maxy = max(cell_pts[:,1])
				maxz = max(cell_pts[:,2])

				minx = min(cell_pts[:,0])
				miny = min(cell_pts[:,1])
				minz = min(cell_pts[:,2])

				max_xyz = max(maxx,maxy,maxz)
				min_xyz = min(minx,miny,minz)

				maxx = max_xyz
				maxy = max_xyz
				maxz = max_xyz
				minx = min_xyz
				miny = min_xyz
				minz = min_xyz

				pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
				opts_scale = np.column_stack(((opts[:,0]-minx)/(maxx-minx),(opts[:,1]-miny)/(maxy-miny),(opts[:,2]-minz)/(maxz-minz)))
				cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
				nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

				cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts_scale])

				quads.append(QuadScheme(cur_pat,100))
				patdists.append(patDist(quads[-1]))
				
			likes = []
			betas = []
			failed = []
			for qind in xrange(0,len(quads)):
				q = quads[qind]
				try:
					beta, loglike = FitModel( q , covar=patdists[qind])
					likes.append(loglike)
					betas.append(beta)
				except:
					failed.append(qind)
					pass


			fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

			with open(beta_file,'w') as f:
				for val in betas:
					for b in val:
						f.write(str(b)+' ')
					f.write('\n')

			cv_ll = []
			for qind in xrange(0,len(fit_quads)):
				held = fit_quads[qind]
				mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
				logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
				# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
				cv_ll.append(logl)


				with open(res_file,'a') as f:
					f.write(str(logl)+'\n')

			avg_ll = np.mean(cv_ll)
			var_ll = np.var(cv_ll)
			with open(res_file,'a') as f:
				f.write('\n')
				f.write('mean: '+str(avg_ll)+'\n')
				f.write('var: '+str(var_ll)+'\n')

			# os.remove(tmp_file)

def inHomo_n_pat(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [nucDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:

			tmp_file = res_dir+'/'+rna1+'_'+rna2+'.tmp'
			res_file = res_dir+'/'+rna1+'_'+rna2+'.txt'
			beta_file = res_dir+'/'+rna1+'_'+rna2+'_params.txt'

			if os.path.exists(tmp_file) or os.path.exists(res_file):
				continue
			else:
				out = open(tmp_file, 'w')
				out.write("lol")
				out.close()

				out = open(res_file, 'w')
				out.close()

			probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2]]
			rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
			rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
			
			
			quads = []
			# cdists = []
			# ndists = []
			fdists = []
			patdists = []
			img_idents = []
			for pfind in xrange(0,len(rna1_files)):
				ptfile = rna1_files[pfind]
				img_pref = ptfile.split('prot')[0]
				img_idents.append(img_pref)
				optfile = img_pref+'prot'+rna2+'.txt'

				pts = np.loadtxt(dat_dir+ptfile, dtype=float)
				opts = np.loadtxt(dat_dir+optfile, dtype=float)
				
				cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
				nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

				maxx = max(cell_pts[:,0])
				maxy = max(cell_pts[:,1])
				maxz = max(cell_pts[:,2])

				minx = min(cell_pts[:,0])
				miny = min(cell_pts[:,1])
				minz = min(cell_pts[:,2])

				max_xyz = max(maxx,maxy,maxz)
				min_xyz = min(minx,miny,minz)

				maxx = max_xyz
				maxy = max_xyz
				maxz = max_xyz
				minx = min_xyz
				miny = min_xyz
				minz = min_xyz

				pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
				opts_scale = np.column_stack(((opts[:,0]-minx)/(maxx-minx),(opts[:,1]-miny)/(maxy-miny),(opts[:,2]-minz)/(maxz-minz)))
				cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
				nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
				cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts_scale])
				quads.append(QuadScheme(cur_pat,100))
				# cdists.append(cellDist(quads[-1]))
				# ndists.append(nucDist(quads[-1]))
				fdists.append(nucDist(quads[-1]))
				patdists.append(patDist(quads[-1]))
				
			likes = []
			betas = []
			failed = []
			for qind in xrange(0,len(quads)):
				q = quads[qind]
				try:
					# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
					beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
					likes.append(loglike)
					betas.append(beta)
				except:
					failed.append(qind)
					pass


			fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]
			fit_idents = [ident for i,ident in enumerate(img_idents) if i not in failed]

			with open(beta_file,'w') as f:
				for val in betas:
					for b in val:
						f.write(str(b)+' ')
					f.write('\n')

			cv_ll = []
			held_idents = []
			for qind in xrange(0,len(fit_quads)):
				held = fit_quads[qind]
				mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
				logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
				# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
				cv_ll.append(logl)
				h_id = fit_idents[qind]


				with open(res_file,'a') as f:
					f.write(str(h_id)+', '+str(logl)+'\n')

			avg_ll = np.mean(cv_ll)
			var_ll = np.var(cv_ll)
			with open(res_file,'a') as f:
				f.write('\n')
				f.write('mean: '+str(avg_ll)+'\n')
				f.write('var: '+str(var_ll)+'\n')

			os.remove(tmp_file)

def inHomo_c_pat(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [cellDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:

			tmp_file = res_dir+'/'+rna1+'_'+rna2+'.tmp'
			res_file = res_dir+'/'+rna1+'_'+rna2+'.txt'
			beta_file = res_dir+'/'+rna1+'_'+rna2+'_params.txt'

			if os.path.exists(tmp_file) or os.path.exists(res_file):
				continue
			else:
				out = open(tmp_file, 'w')
				out.write("lol")
				out.close()

				out = open(res_file, 'w')
				out.close()

			probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2]]
			rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
			rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
			
			
			quads = []
			# cdists = []
			# ndists = []
			fdists = []
			patdists = []
			img_idents = []
			for pfind in xrange(0,len(rna1_files)):
				ptfile = rna1_files[pfind]
				img_pref = ptfile.split('prot')[0]
				img_idents.append(img_pref)
				optfile = img_pref+'prot'+rna2+'.txt'

				pts = np.loadtxt(dat_dir+ptfile, dtype=float)
				opts = np.loadtxt(dat_dir+optfile, dtype=float)
				
				cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
				nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

				maxx = max(cell_pts[:,0])
				maxy = max(cell_pts[:,1])
				maxz = max(cell_pts[:,2])

				minx = min(cell_pts[:,0])
				miny = min(cell_pts[:,1])
				minz = min(cell_pts[:,2])

				max_xyz = max(maxx,maxy,maxz)
				min_xyz = min(minx,miny,minz)

				maxx = max_xyz
				maxy = max_xyz
				maxz = max_xyz
				minx = min_xyz
				miny = min_xyz
				minz = min_xyz

				pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
				opts_scale = np.column_stack(((opts[:,0]-minx)/(maxx-minx),(opts[:,1]-miny)/(maxy-miny),(opts[:,2]-minz)/(maxz-minz)))
				cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
				nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
				cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts_scale])
				quads.append(QuadScheme(cur_pat,100))
				# cdists.append(cellDist(quads[-1]))
				# ndists.append(nucDist(quads[-1]))
				fdists.append(cellDist(quads[-1]))
				patdists.append(patDist(quads[-1]))
				
			likes = []
			betas = []
			failed = []
			for qind in xrange(0,len(quads)):
				q = quads[qind]
				try:
					# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
					beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
					likes.append(loglike)
					betas.append(beta)
				except:
					failed.append(qind)
					pass


			fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]
			fit_idents = [ident for i,ident in enumerate(img_idents) if i not in failed]

			with open(beta_file,'w') as f:
				for val in betas:
					for b in val:
						f.write(str(b)+' ')
					f.write('\n')

			cv_ll = []
			held_idents = []
			for qind in xrange(0,len(fit_quads)):
				held = fit_quads[qind]
				mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
				logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
				# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
				cv_ll.append(logl)
				h_id = fit_idents[qind]


				with open(res_file,'a') as f:
					f.write(str(h_id)+', '+str(logl)+'\n')

			avg_ll = np.mean(cv_ll)
			var_ll = np.var(cv_ll)
			with open(res_file,'a') as f:
				f.write('\n')
				f.write('mean: '+str(avg_ll)+'\n')
				f.write('var: '+str(var_ll)+'\n')

			os.remove(tmp_file)
## inhomogenous, dependent on distance to one other pattern and cell/nuclear ratio
def inHomo_cn_pat(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [fracDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:

			tmp_file = res_dir+'/'+rna1+'_'+rna2+'.tmp'
			res_file = res_dir+'/'+rna1+'_'+rna2+'.txt'
			beta_file = res_dir+'/'+rna1+'_'+rna2+'_params.txt'

			if os.path.exists(tmp_file) or os.path.exists(res_file):
				continue
			else:
				out = open(tmp_file, 'w')
				out.write("lol")
				out.close()

				out = open(res_file, 'w')
				out.close()

			probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2]]
			rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
			rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
			
			
			quads = []
			# cdists = []
			# ndists = []
			fdists = []
			patdists = []
			img_idents = []
			for pfind in xrange(0,len(rna1_files)):
				ptfile = rna1_files[pfind]
				img_pref = ptfile.split('prot')[0]
				img_idents.append(img_pref)
				optfile = img_pref+'prot'+rna2+'.txt'

				pts = np.loadtxt(dat_dir+ptfile, dtype=float)
				opts = np.loadtxt(dat_dir+optfile, dtype=float)
				
				cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
				nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

				maxx = max(cell_pts[:,0])
				maxy = max(cell_pts[:,1])
				maxz = max(cell_pts[:,2])

				minx = min(cell_pts[:,0])
				miny = min(cell_pts[:,1])
				minz = min(cell_pts[:,2])

				max_xyz = max(maxx,maxy,maxz)
				min_xyz = min(minx,miny,minz)

				maxx = max_xyz
				maxy = max_xyz
				maxz = max_xyz
				minx = min_xyz
				miny = min_xyz
				minz = min_xyz

				pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
				opts_scale = np.column_stack(((opts[:,0]-minx)/(maxx-minx),(opts[:,1]-miny)/(maxy-miny),(opts[:,2]-minz)/(maxz-minz)))
				cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
				nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
				cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts_scale])
				quads.append(QuadScheme(cur_pat,100))
				# cdists.append(cellDist(quads[-1]))
				# ndists.append(nucDist(quads[-1]))
				fdists.append(fracDist(quads[-1]))
				patdists.append(patDist(quads[-1]))
				
			likes = []
			betas = []
			failed = []
			for qind in xrange(0,len(quads)):
				q = quads[qind]
				try:
					# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
					beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
					likes.append(loglike)
					betas.append(beta)
				except:
					failed.append(qind)
					pass


			fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]
			fit_idents = [ident for i,ident in enumerate(img_idents) if i not in failed]

			with open(beta_file,'w') as f:
				for val in betas:
					for b in val:
						f.write(str(b)+' ')
					f.write('\n')

			cv_ll = []
			held_idents = []
			for qind in xrange(0,len(fit_quads)):
				held = fit_quads[qind]
				mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
				logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
				# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
				cv_ll.append(logl)
				h_id = fit_idents[qind]


				with open(res_file,'a') as f:
					f.write(str(h_id)+', '+str(logl)+'\n')

			avg_ll = np.mean(cv_ll)
			var_ll = np.var(cv_ll)
			with open(res_file,'a') as f:
				f.write('\n')
				f.write('mean: '+str(avg_ll)+'\n')
				f.write('var: '+str(var_ll)+'\n')

			os.remove(tmp_file)

## inhomogenous, dependent on distance to two other pattern and cell/nuclear ratio
def inHomo_tri(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	# cov_funcs = [fracDist,patDist]
	cov_funcs = [patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:

				if rna1 == rna2 or rna1==rna3 or rna2==rna3:
					continue

				

				probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3]]
				
				if probes == []:
					# os.remove(tmp_file)
					continue


				tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.tmp'
				res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.txt'
				beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_params.txt'

				if os.path.exists(tmp_file) or os.path.exists(res_file):
					continue
				else:
					out = open(tmp_file, 'w')
					out.write("lol")
					out.close()

					out = open(res_file, 'w')
					out.close()
					
				rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
				rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
				rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
				
				
				quads = []
				# cdists = []
				# ndists = []
				# fdists = []
				patdists = []
				for pfind in xrange(0,len(rna1_files)):
					ptfile = rna1_files[pfind]
					img_pref = ptfile.split('prot')[0]
					optfile1 = img_pref+'prot'+rna2+'.txt'
					optfile2 = img_pref+'prot'+rna3+'.txt'

					pts = np.loadtxt(dat_dir+ptfile, dtype=float)
					opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
					opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
					
					cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
					nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

					maxx = max(cell_pts[:,0])
					maxy = max(cell_pts[:,1])
					maxz = max(cell_pts[:,2])

					minx = min(cell_pts[:,0])
					miny = min(cell_pts[:,1])
					minz = min(cell_pts[:,2])

					max_xyz = max(maxx,maxy,maxz)
					min_xyz = min(minx,miny,minz)

					maxx = max_xyz
					maxy = max_xyz
					maxz = max_xyz
					minx = min_xyz
					miny = min_xyz
					minz = min_xyz

					pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
					opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
					opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
					cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
					nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
			
					cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale])




					quads.append(QuadScheme(cur_pat,50))
					# cdists.append(cellDist(quads[-1]))
					# ndists.append(nucDist(quads[-1]))
					# fdists.append(fracDist(quads[-1]))
					patdists.append(patDist(quads[-1]))
					
				likes = []
				betas = []
				failed = []
				for qind in xrange(0,len(quads)):
					q = quads[qind]
					try:
						# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
						# beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
						beta, loglike = FitModel( q , covar=patdists[qind] )
						likes.append(loglike)
						betas.append(beta)
					except:
						failed.append(qind)
						pass


				fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

				with open(beta_file,'w') as f:
					for val in betas:
						for b in val:
							f.write(str(b)+' ')
						f.write('\n')

				cv_ll = []
				for qind in xrange(0,len(fit_quads)):
					held = fit_quads[qind]
					mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
					logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
					# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
					cv_ll.append(logl)


					with open(res_file,'a') as f:
						f.write(str(logl)+'\n')

				avg_ll = np.mean(cv_ll)
				var_ll = np.var(cv_ll)
				with open(res_file,'a') as f:
					f.write('\n')
					f.write('mean: '+str(avg_ll)+'\n')
					f.write('var: '+str(var_ll)+'\n')

				# os.remove(tmp_file)

def inHomo_n_tri(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [nucDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:

				if rna1 == rna2 or rna1==rna3 or rna2==rna3:
					continue

				

				probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3]]
				
				if probes == []:
					# os.remove(tmp_file)
					continue


				tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.tmp'
				res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.txt'
				beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_params.txt'

				if os.path.exists(tmp_file) or os.path.exists(res_file):
					continue
				else:
					out = open(tmp_file, 'w')
					out.write("lol")
					out.close()

					out = open(res_file, 'w')
					out.close()
					
				rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
				rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
				rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
				
				
				quads = []
				# cdists = []
				# ndists = []
				fdists = []
				patdists = []
				for pfind in xrange(0,len(rna1_files)):
					ptfile = rna1_files[pfind]
					img_pref = ptfile.split('prot')[0]
					optfile1 = img_pref+'prot'+rna2+'.txt'
					optfile2 = img_pref+'prot'+rna3+'.txt'

					pts = np.loadtxt(dat_dir+ptfile, dtype=float)
					opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
					opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
					
					cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
					nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

					maxx = max(cell_pts[:,0])
					maxy = max(cell_pts[:,1])
					maxz = max(cell_pts[:,2])

					minx = min(cell_pts[:,0])
					miny = min(cell_pts[:,1])
					minz = min(cell_pts[:,2])

					max_xyz = max(maxx,maxy,maxz)
					min_xyz = min(minx,miny,minz)

					maxx = max_xyz
					maxy = max_xyz
					maxz = max_xyz
					minx = min_xyz
					miny = min_xyz
					minz = min_xyz

					pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
					opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
					opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
					cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
					nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
			
					cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale])




					quads.append(QuadScheme(cur_pat,50))
					# cdists.append(cellDist(quads[-1]))
					# ndists.append(nucDist(quads[-1]))
					fdists.append(nucDist(quads[-1]))
					patdists.append(patDist(quads[-1]))
					
				likes = []
				betas = []
				failed = []
				for qind in xrange(0,len(quads)):
					q = quads[qind]
					try:
						# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
						beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
						likes.append(loglike)
						betas.append(beta)
					except:
						failed.append(qind)
						pass


				fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

				with open(beta_file,'w') as f:
					for val in betas:
						for b in val:
							f.write(str(b)+' ')
						f.write('\n')

				cv_ll = []
				for qind in xrange(0,len(fit_quads)):
					held = fit_quads[qind]
					mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
					logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
					# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
					cv_ll.append(logl)


					with open(res_file,'a') as f:
						f.write(str(logl)+'\n')

				avg_ll = np.mean(cv_ll)
				var_ll = np.var(cv_ll)
				with open(res_file,'a') as f:
					f.write('\n')
					f.write('mean: '+str(avg_ll)+'\n')
					f.write('var: '+str(var_ll)+'\n')

def inHomo_c_tri(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [cellDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:

				if rna1 == rna2 or rna1==rna3 or rna2==rna3:
					continue

				

				probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3]]
				
				if probes == []:
					# os.remove(tmp_file)
					continue


				tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.tmp'
				res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.txt'
				beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_params.txt'

				if os.path.exists(tmp_file) or os.path.exists(res_file):
					continue
				else:
					out = open(tmp_file, 'w')
					out.write("lol")
					out.close()

					out = open(res_file, 'w')
					out.close()
					
				rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
				rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
				rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
				
				
				quads = []
				# cdists = []
				# ndists = []
				fdists = []
				patdists = []
				for pfind in xrange(0,len(rna1_files)):
					ptfile = rna1_files[pfind]
					img_pref = ptfile.split('prot')[0]
					optfile1 = img_pref+'prot'+rna2+'.txt'
					optfile2 = img_pref+'prot'+rna3+'.txt'

					pts = np.loadtxt(dat_dir+ptfile, dtype=float)
					opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
					opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
					
					cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
					nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

					maxx = max(cell_pts[:,0])
					maxy = max(cell_pts[:,1])
					maxz = max(cell_pts[:,2])

					minx = min(cell_pts[:,0])
					miny = min(cell_pts[:,1])
					minz = min(cell_pts[:,2])

					max_xyz = max(maxx,maxy,maxz)
					min_xyz = min(minx,miny,minz)

					maxx = max_xyz
					maxy = max_xyz
					maxz = max_xyz
					minx = min_xyz
					miny = min_xyz
					minz = min_xyz

					pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
					opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
					opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
					cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
					nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
			
					cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale])




					quads.append(QuadScheme(cur_pat,50))
					# cdists.append(cellDist(quads[-1]))
					# ndists.append(nucDist(quads[-1]))
					fdists.append(cellDist(quads[-1]))
					patdists.append(patDist(quads[-1]))
					
				likes = []
				betas = []
				failed = []
				for qind in xrange(0,len(quads)):
					q = quads[qind]
					try:
						# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
						beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
						likes.append(loglike)
						betas.append(beta)
					except:
						failed.append(qind)
						pass


				fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

				with open(beta_file,'w') as f:
					for val in betas:
						for b in val:
							f.write(str(b)+' ')
						f.write('\n')

				cv_ll = []
				for qind in xrange(0,len(fit_quads)):
					held = fit_quads[qind]
					mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
					logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
					# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
					cv_ll.append(logl)


					with open(res_file,'a') as f:
						f.write(str(logl)+'\n')

				avg_ll = np.mean(cv_ll)
				var_ll = np.var(cv_ll)
				with open(res_file,'a') as f:
					f.write('\n')
					f.write('mean: '+str(avg_ll)+'\n')
					f.write('var: '+str(var_ll)+'\n')

## inhomogenous, dependent on distance to two other patterns and cell/nuclear ratio
def inHomo_cn_tri(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist]
	cov_funcs = [fracDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:

				if rna1 == rna2 or rna1==rna3 or rna2==rna3:
					continue

				

				probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3]]
				
				if probes == []:
					# os.remove(tmp_file)
					continue


				tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.tmp'
				res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'.txt'
				beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_params.txt'

				if os.path.exists(tmp_file) or os.path.exists(res_file):
					continue
				else:
					out = open(tmp_file, 'w')
					out.write("lol")
					out.close()

					out = open(res_file, 'w')
					out.close()
					
				rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
				rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
				rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
				
				
				quads = []
				# cdists = []
				# ndists = []
				fdists = []
				patdists = []
				for pfind in xrange(0,len(rna1_files)):
					ptfile = rna1_files[pfind]
					img_pref = ptfile.split('prot')[0]
					optfile1 = img_pref+'prot'+rna2+'.txt'
					optfile2 = img_pref+'prot'+rna3+'.txt'

					pts = np.loadtxt(dat_dir+ptfile, dtype=float)
					opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
					opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
					
					cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
					nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

					maxx = max(cell_pts[:,0])
					maxy = max(cell_pts[:,1])
					maxz = max(cell_pts[:,2])

					minx = min(cell_pts[:,0])
					miny = min(cell_pts[:,1])
					minz = min(cell_pts[:,2])

					max_xyz = max(maxx,maxy,maxz)
					min_xyz = min(minx,miny,minz)

					maxx = max_xyz
					maxy = max_xyz
					maxz = max_xyz
					minx = min_xyz
					miny = min_xyz
					minz = min_xyz

					pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
					opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
					opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
					cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
					nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))
			
					cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale])




					quads.append(QuadScheme(cur_pat,50))
					# cdists.append(cellDist(quads[-1]))
					# ndists.append(nucDist(quads[-1]))
					fdists.append(fracDist(quads[-1]))
					patdists.append(patDist(quads[-1]))
					
				likes = []
				betas = []
				failed = []
				for qind in xrange(0,len(quads)):
					q = quads[qind]
					try:
						# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
						beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
						likes.append(loglike)
						betas.append(beta)
					except:
						failed.append(qind)
						pass


				fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

				with open(beta_file,'w') as f:
					for val in betas:
						for b in val:
							f.write(str(b)+' ')
						f.write('\n')

				cv_ll = []
				for qind in xrange(0,len(fit_quads)):
					held = fit_quads[qind]
					mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
					logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
					# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
					cv_ll.append(logl)


					with open(res_file,'a') as f:
						f.write(str(logl)+'\n')

				avg_ll = np.mean(cv_ll)
				var_ll = np.var(cv_ll)
				with open(res_file,'a') as f:
					f.write('\n')
					f.write('mean: '+str(avg_ll)+'\n')
					f.write('var: '+str(var_ll)+'\n')

				# os.remove(tmp_file)

## inhomogenous, dependent on distance to three other patterns
def inHomo_quad(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist,patDist,patDist]
	cov_funcs = [patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:
				for rna4 in rna_idents:

					if rna1 == rna2 or rna1==rna3 or rna1 == rna4 or rna2==rna3 or rna2 == rna4 or rna3 == rna4:
						continue

					

					probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3] and probe in probes_per_rna[rna4]]
					
					if probes == []:
						# os.remove(tmp_file)
						# os.remove(res_file)
						continue

					tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.tmp'
					res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.txt'
					beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'_params.txt'

					if os.path.exists(tmp_file) or os.path.exists(res_file):
						continue
					else:
						out = open(tmp_file, 'w')
						out.write("lol")
						out.close()

						out = open(res_file, 'w')
						out.close()
						
					rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
					rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
					rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
					rna4_files = [f for f in rna_files[rna4] if any(f.startswith(probe) for probe in probes)]
					
					
					quads = []
					# cdists = []
					# ndists = []
					# fdists = []
					patdists = []
					for pfind in xrange(0,len(rna1_files)):
						ptfile = rna1_files[pfind]
						img_pref = ptfile.split('prot')[0]
						optfile1 = img_pref+'prot'+rna2+'.txt'
						optfile2 = img_pref+'prot'+rna3+'.txt'
						optfile3 = img_pref+'prot'+rna4+'.txt'

						pts = np.loadtxt(dat_dir+ptfile, dtype=float)
						opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
						opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
						opts3 = np.loadtxt(dat_dir+optfile3, dtype=float)
						cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
						nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

						maxx = max(cell_pts[:,0])
						maxy = max(cell_pts[:,1])
						maxz = max(cell_pts[:,2])

						minx = min(cell_pts[:,0])
						miny = min(cell_pts[:,1])
						minz = min(cell_pts[:,2])

						max_xyz = max(maxx,maxy,maxz)
						min_xyz = min(minx,miny,minz)

						maxx = max_xyz
						maxy = max_xyz
						maxz = max_xyz
						minx = min_xyz
						miny = min_xyz
						minz = min_xyz

						pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
						opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
						opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
						opts3_scale = np.column_stack(((opts3[:,0]-minx)/(maxx-minx),(opts3[:,1]-miny)/(maxy-miny),(opts3[:,2]-minz)/(maxz-minz)))
						cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
						nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

						
						

	########### need to change spp.py to support multiple other patterns as list ##############					
						cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale,opts3_scale])

						quads.append(QuadScheme(cur_pat,50))
						# cdists.append(cellDist(quads[-1]))
						# ndists.append(nucDist(quads[-1]))
						# fdists.append(fracDist(quads[-1]))
						patdists.append(patDist(quads[-1]))
						
					likes = []
					betas = []
					failed = []
					for qind in xrange(0,len(quads)):
						q = quads[qind]
						try:
							# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
							# beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
							beta, loglike = FitModel( q , covar=patdists[qind] )
							likes.append(loglike)
							betas.append(beta)
						except:
							failed.append(qind)
							pass


					fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

					with open(beta_file,'w') as f:
						for val in betas:
							for b in val:
								f.write(str(b)+' ')
							f.write('\n')

					cv_ll = []
					for qind in xrange(0,len(fit_quads)):
						held = fit_quads[qind]
						mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
						logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
						# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
						cv_ll.append(logl)


						with open(res_file,'a') as f:
							f.write(str(logl)+'\n')

					avg_ll = np.mean(cv_ll)
					var_ll = np.var(cv_ll)
					with open(res_file,'a') as f:
						f.write('\n')
						f.write('mean: '+str(avg_ll)+'\n')
						f.write('var: '+str(var_ll)+'\n')

					# os.remove(tmp_file)

## inhomogenous, dependent on distance to three other patterns and cell/nuclear ratio
def inHomo_n_quad(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist,patDist,patDist]
	cov_funcs = [nucDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:
				for rna4 in rna_idents:

					if rna1 == rna2 or rna1==rna3 or rna1 == rna4 or rna2==rna3 or rna2 == rna4 or rna3 == rna4:
						continue

					

					probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3] and probe in probes_per_rna[rna4]]
					
					if probes == []:
						# os.remove(tmp_file)
						# os.remove(res_file)
						continue

					tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.tmp'
					res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.txt'
					beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'_params.txt'

					if os.path.exists(tmp_file) or os.path.exists(res_file):
						continue
					else:
						out = open(tmp_file, 'w')
						out.write("lol")
						out.close()

						out = open(res_file, 'w')
						out.close()
						
					rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
					rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
					rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
					rna4_files = [f for f in rna_files[rna4] if any(f.startswith(probe) for probe in probes)]
					
					
					quads = []
					# cdists = []
					# ndists = []
					fdists = []
					patdists = []
					for pfind in xrange(0,len(rna1_files)):
						ptfile = rna1_files[pfind]
						img_pref = ptfile.split('prot')[0]
						optfile1 = img_pref+'prot'+rna2+'.txt'
						optfile2 = img_pref+'prot'+rna3+'.txt'
						optfile3 = img_pref+'prot'+rna4+'.txt'

						pts = np.loadtxt(dat_dir+ptfile, dtype=float)
						opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
						opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
						opts3 = np.loadtxt(dat_dir+optfile3, dtype=float)
						cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
						nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

						maxx = max(cell_pts[:,0])
						maxy = max(cell_pts[:,1])
						maxz = max(cell_pts[:,2])

						minx = min(cell_pts[:,0])
						miny = min(cell_pts[:,1])
						minz = min(cell_pts[:,2])

						max_xyz = max(maxx,maxy,maxz)
						min_xyz = min(minx,miny,minz)

						maxx = max_xyz
						maxy = max_xyz
						maxz = max_xyz
						minx = min_xyz
						miny = min_xyz
						minz = min_xyz

						pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
						opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
						opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
						opts3_scale = np.column_stack(((opts3[:,0]-minx)/(maxx-minx),(opts3[:,1]-miny)/(maxy-miny),(opts3[:,2]-minz)/(maxz-minz)))
						cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
						nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

						
						

	########### need to change spp.py to support multiple other patterns as list ##############					
						cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale,opts3_scale])

						quads.append(QuadScheme(cur_pat,50))
						# cdists.append(cellDist(quads[-1]))
						# ndists.append(nucDist(quads[-1]))
						fdists.append(nucDist(quads[-1]))
						patdists.append(patDist(quads[-1]))
						
					likes = []
					betas = []
					failed = []
					for qind in xrange(0,len(quads)):
						q = quads[qind]
						try:
							# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
							beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
							likes.append(loglike)
							betas.append(beta)
						except:
							failed.append(qind)
							pass


					fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

					with open(beta_file,'w') as f:
						for val in betas:
							for b in val:
								f.write(str(b)+' ')
							f.write('\n')

					cv_ll = []
					for qind in xrange(0,len(fit_quads)):
						held = fit_quads[qind]
						mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
						logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
						# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
						cv_ll.append(logl)


						with open(res_file,'a') as f:
							f.write(str(logl)+'\n')

					avg_ll = np.mean(cv_ll)
					var_ll = np.var(cv_ll)
					with open(res_file,'a') as f:
						f.write('\n')
						f.write('mean: '+str(avg_ll)+'\n')
						f.write('var: '+str(var_ll)+'\n')

					# os.remove(tmp_file)

def inHomo_c_quad(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist,patDist,patDist]
	cov_funcs = [cellDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:
				for rna4 in rna_idents:

					if rna1 == rna2 or rna1==rna3 or rna1 == rna4 or rna2==rna3 or rna2 == rna4 or rna3 == rna4:
						continue

					

					probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3] and probe in probes_per_rna[rna4]]
					
					if probes == []:
						# os.remove(tmp_file)
						# os.remove(res_file)
						continue

					tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.tmp'
					res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.txt'
					beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'_params.txt'

					if os.path.exists(tmp_file) or os.path.exists(res_file):
						continue
					else:
						out = open(tmp_file, 'w')
						out.write("lol")
						out.close()

						out = open(res_file, 'w')
						out.close()
						
					rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
					rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
					rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
					rna4_files = [f for f in rna_files[rna4] if any(f.startswith(probe) for probe in probes)]
					
					
					quads = []
					# cdists = []
					# ndists = []
					fdists = []
					patdists = []
					for pfind in xrange(0,len(rna1_files)):
						ptfile = rna1_files[pfind]
						img_pref = ptfile.split('prot')[0]
						optfile1 = img_pref+'prot'+rna2+'.txt'
						optfile2 = img_pref+'prot'+rna3+'.txt'
						optfile3 = img_pref+'prot'+rna4+'.txt'

						pts = np.loadtxt(dat_dir+ptfile, dtype=float)
						opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
						opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
						opts3 = np.loadtxt(dat_dir+optfile3, dtype=float)
						cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
						nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

						maxx = max(cell_pts[:,0])
						maxy = max(cell_pts[:,1])
						maxz = max(cell_pts[:,2])

						minx = min(cell_pts[:,0])
						miny = min(cell_pts[:,1])
						minz = min(cell_pts[:,2])

						max_xyz = max(maxx,maxy,maxz)
						min_xyz = min(minx,miny,minz)

						maxx = max_xyz
						maxy = max_xyz
						maxz = max_xyz
						minx = min_xyz
						miny = min_xyz
						minz = min_xyz

						pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
						opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
						opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
						opts3_scale = np.column_stack(((opts3[:,0]-minx)/(maxx-minx),(opts3[:,1]-miny)/(maxy-miny),(opts3[:,2]-minz)/(maxz-minz)))
						cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
						nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

						
						

	########### need to change spp.py to support multiple other patterns as list ##############					
						cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale,opts3_scale])

						quads.append(QuadScheme(cur_pat,50))
						# cdists.append(cellDist(quads[-1]))
						# ndists.append(nucDist(quads[-1]))
						fdists.append(cellDist(quads[-1]))
						patdists.append(patDist(quads[-1]))
						
					likes = []
					betas = []
					failed = []
					for qind in xrange(0,len(quads)):
						q = quads[qind]
						try:
							# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
							beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
							likes.append(loglike)
							betas.append(beta)
						except:
							failed.append(qind)
							pass


					fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

					with open(beta_file,'w') as f:
						for val in betas:
							for b in val:
								f.write(str(b)+' ')
							f.write('\n')

					cv_ll = []
					for qind in xrange(0,len(fit_quads)):
						held = fit_quads[qind]
						mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
						logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
						# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
						cv_ll.append(logl)


						with open(res_file,'a') as f:
							f.write(str(logl)+'\n')

					avg_ll = np.mean(cv_ll)
					var_ll = np.var(cv_ll)
					with open(res_file,'a') as f:
						f.write('\n')
						f.write('mean: '+str(avg_ll)+'\n')
						f.write('var: '+str(var_ll)+'\n')

					# os.remove(tmp_file)

def inHomo_cn_quad(base_dir, dat_dir, res_dir, rna_idents, rna_files, probes_per_rna):
	# cov_funcs = [cellDist,nucDist,patDist,patDist,patDist]
	cov_funcs = [fracDist,patDist]
	for rna1 in rna_idents:
		for rna2 in rna_idents:
			for rna3 in rna_idents:
				for rna4 in rna_idents:

					if rna1 == rna2 or rna1==rna3 or rna1 == rna4 or rna2==rna3 or rna2 == rna4 or rna3 == rna4:
						continue

					

					probes = [probe.lower() for probe in probes_per_rna[rna1] if probe in probes_per_rna[rna2] and probe in probes_per_rna[rna3] and probe in probes_per_rna[rna4]]
					
					if probes == []:
						# os.remove(tmp_file)
						# os.remove(res_file)
						continue

					tmp_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.tmp'
					res_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'.txt'
					beta_file = res_dir+'/'+rna1+'_'+rna2+'_'+rna3+'_'+rna4+'_params.txt'

					if os.path.exists(tmp_file) or os.path.exists(res_file):
						continue
					else:
						out = open(tmp_file, 'w')
						out.write("lol")
						out.close()

						out = open(res_file, 'w')
						out.close()
						
					rna1_files = [f for f in rna_files[rna1] if any(f.startswith(probe) for probe in probes)]
					rna2_files = [f for f in rna_files[rna2] if any(f.startswith(probe) for probe in probes)]
					rna3_files = [f for f in rna_files[rna3] if any(f.startswith(probe) for probe in probes)]
					rna4_files = [f for f in rna_files[rna4] if any(f.startswith(probe) for probe in probes)]
					
					
					quads = []
					# cdists = []
					# ndists = []
					fdists = []
					patdists = []
					for pfind in xrange(0,len(rna1_files)):
						ptfile = rna1_files[pfind]
						img_pref = ptfile.split('prot')[0]
						optfile1 = img_pref+'prot'+rna2+'.txt'
						optfile2 = img_pref+'prot'+rna3+'.txt'
						optfile3 = img_pref+'prot'+rna4+'.txt'

						pts = np.loadtxt(dat_dir+ptfile, dtype=float)
						opts1 = np.loadtxt(dat_dir+optfile1, dtype=float)
						opts2 = np.loadtxt(dat_dir+optfile2, dtype=float)
						opts3 = np.loadtxt(dat_dir+optfile3, dtype=float)
						cell_pts = np.loadtxt(dat_dir+img_pref+'cell.txt',dtype=float)
						nuc_pts = np.loadtxt(dat_dir+img_pref+'nuc.txt',dtype=float)

						maxx = max(cell_pts[:,0])
						maxy = max(cell_pts[:,1])
						maxz = max(cell_pts[:,2])

						minx = min(cell_pts[:,0])
						miny = min(cell_pts[:,1])
						minz = min(cell_pts[:,2])

						max_xyz = max(maxx,maxy,maxz)
						min_xyz = min(minx,miny,minz)

						maxx = max_xyz
						maxy = max_xyz
						maxz = max_xyz
						minx = min_xyz
						miny = min_xyz
						minz = min_xyz

						pts_scale = np.column_stack(((pts[:,0]-minx)/(maxx-minx),(pts[:,1]-miny)/(maxy-miny),(pts[:,2]-minz)/(maxz-minz)))
						opts1_scale = np.column_stack(((opts1[:,0]-minx)/(maxx-minx),(opts1[:,1]-miny)/(maxy-miny),(opts1[:,2]-minz)/(maxz-minz)))
						opts2_scale = np.column_stack(((opts2[:,0]-minx)/(maxx-minx),(opts2[:,1]-miny)/(maxy-miny),(opts2[:,2]-minz)/(maxz-minz)))
						opts3_scale = np.column_stack(((opts3[:,0]-minx)/(maxx-minx),(opts3[:,1]-miny)/(maxy-miny),(opts3[:,2]-minz)/(maxz-minz)))
						cell_pts_scale = np.column_stack(((cell_pts[:,0]-minx)/(maxx-minx),(cell_pts[:,1]-miny)/(maxy-miny),(cell_pts[:,2]-minz)/(maxz-minz)))
						nuc_pts_scale = np.column_stack(((nuc_pts[:,0]-minx)/(maxx-minx),(nuc_pts[:,1]-miny)/(maxy-miny),(nuc_pts[:,2]-minz)/(maxz-minz)))

						
						

	########### need to change spp.py to support multiple other patterns as list ##############					
						cur_pat = PointPattern(pts_scale,mark=rna1,cellpat=cell_pts_scale,nucpat=nuc_pts_scale,dimension=3,otherpat=[opts1_scale,opts2_scale,opts3_scale])

						quads.append(QuadScheme(cur_pat,50))
						# cdists.append(cellDist(quads[-1]))
						# ndists.append(nucDist(quads[-1]))
						fdists.append(fracDist(quads[-1]))
						patdists.append(patDist(quads[-1]))
						
					likes = []
					betas = []
					failed = []
					for qind in xrange(0,len(quads)):
						q = quads[qind]
						try:
							# beta, loglike = FitModel( q , covar=np.column_stack((cdists[qind],ndists[qind],patdists[qind])))
							beta, loglike = FitModel( q , covar=np.column_stack((fdists[qind],patdists[qind])))
							likes.append(loglike)
							betas.append(beta)
						except:
							failed.append(qind)
							pass


					fit_quads = [quad for qind,quad in enumerate(quads) if qind not in failed]

					with open(beta_file,'w') as f:
						for val in betas:
							for b in val:
								f.write(str(b)+' ')
							f.write('\n')

					cv_ll = []
					for qind in xrange(0,len(fit_quads)):
						held = fit_quads[qind]
						mean_beta = np.mean([betas[bind] for bind in xrange(0,len(betas)) if bind != qind],axis=0)
						logl = inhomoLogLike(held.pat,mean_beta,cov_funcs)
						# logl = lpnorm(held.pat,np.divide(cdists[qind][held.ptlabs==1],cdists[qind][held.ptlabs==1]),mean_beta)
						cv_ll.append(logl)


						with open(res_file,'a') as f:
							f.write(str(logl)+'\n')

					avg_ll = np.mean(cv_ll)
					var_ll = np.var(cv_ll)
					with open(res_file,'a') as f:
						f.write('\n')
						f.write('mean: '+str(avg_ll)+'\n')
						f.write('var: '+str(var_ll)+'\n')

					# os.remove(tmp_file)