# network contruction

import os
import itertools
import numpy as np

genKeys_norep = lambda rnas,order: list(itertools.combinations(rnas,order))
genKeys = lambda rnas,order: list(itertools.permutations(rnas,order))

def addNoise(edge_dict, sd):
	"""
	Helper function to add noise to graph
	"""
	noise_dict = {}
	for e in edge_dict:
		noise_dict[e] = edge_dict[e] + np.random.normal(0, sd)
	return noise_dict

def genKeySeries(t,s):
	"""
	Helper function to generate all keys in graph
	"""
	keys = []
	if len(s) == 1:
		nk = genKeys_norep(s,1)
		# rk = [t+k for k in nk]
		rk = [t+k for k in sorted(nk)]
		keys.extend(rk)
	else:
		for i in xrange(1,len(s)):
			nk = genKeys_norep(s,i)
			# rk = [t+k for k in nk]
			rk = [t+k for k in sorted(nk)]
			keys.extend(rk)
	return keys

def partition(collection):
	"""
	Helper function to generate all partitions of a node
	"""
	if len(collection) == 1:
		yield [ collection ]
		return
	first = collection[0]
	for smaller in partition(collection[1:]):
		for n, subset in enumerate(smaller):
			yield smaller[:n] + [[ first ] + subset]  + smaller[n+1:]
		yield [ [ first ] ] + smaller

# def parts(collection):

def mergeDicts(*dicts):
	"""
	Helper function to merge dictionaries together
	"""
	d = {}
	for di in dicts:
		for key in di:
			if key in d and di[key] < d[key]:
				pass
			else:
				d[key] = di[key]
	return d

def getKeys(k_set):
	"""
	Helper function to get all the node keys that we need
	"""
	keys = [(k,) for k in k_set]
	new_keys = [(k,) for k in k_set]
	for num in xrange(2,len(new_keys)+1):
		for f_key in new_keys:
			tmp_keys = genKeys_norep(set([k for k in k_set if k is not f_key[0]]),num-1)
			r_keys = [f_key+tuple(sorted(t)) for t in tmp_keys]
			keys.extend(r_keys)
	return keys

def solveTree(rnas, edge_dict, sd=0):
	"""
	Dynamic programming to find min tree
	"""
	# generate all possible nodes in the graph
	keys = getKeys(rnas)
	final_node = sorted(keys)[0]
	# define the score array, has score of inf for each node
	scores = dict()
	for i in keys:
		if i in edge_dict:
			if sd != 0:
				n_val = -np.abs(np.random.normal(0, sd))
				scores[i] = edge_dict[i] + n_val
			else:
				scores[i] = edge_dict[i]
		elif len(i) == 1:
			scores[i] = 0
		else:
			scores[i] = -np.inf


	# define the indictor array so we can backtrack to find the correct structure
	ind_arr = [-1 for x in keys]

	# main loop
	# for cur_node in keys:
	# while any(scores) == np.inf:
	num_node = 0
	# while any(v == np.inf for v in scores.itervalues()):
		# print num_iter
	for cur_node in keys:
		num_node += 1
		if num_node % 200 == 0:
			print str(num_node)+' / '+str(len(keys))
		if scores[cur_node] == -np.inf:
			# print cur_node
			indicator, score = scoreNode(cur_node,scores,keys,edge_dict)
			scores[cur_node] = score*2
			
			ind_arr[keys.index(cur_node)] = indicator

	# backtrack
	# find 8-mer with highest score:
	e_mers = [k for k in keys if len(k)==len(rnas)]
	scrs = [scores[e] for e in e_mers]
	max_e = e_mers[scrs.index(max(scrs))]
	cur_nodes = [max_e]
	# cur_nodes = [tuple(sorted(['HA', 'PB1', 'PB2', 'NA', 'M', 'PA', 'NP', 'NS']))]
	# cur_nodes = [['HA', 'PB1', 'PB2', 'NA', 'M', 'PA', 'NP', 'NS']]
	
	tree_edges = dict()
	while len(cur_nodes) > 0:
		for node in cur_nodes:
			cur_nodes.remove(node)
			if ind_arr[keys.index(node)] == -1:
				continue
			children_inds = ind_arr[keys.index(node)]
			c_idents = [keys[i] for i in children_inds]
			tree_edges[tuple(node)] = c_idents
			# tree_edges[tuple(node)] = c_idents
			# c_id_n = [keys[i] for i in children_inds if ind_arr[i] != -1]
			c_id_n = [keys[i] for i in children_inds]
			cur_nodes.extend(c_id_n)


	# final_score = scores[tuple(sorted(['HA', 'PB1', 'PB2', 'NA', 'M', 'PA', 'NP', 'NS']))]
	final_score = scores[max_e]

	return scores, ind_arr, tree_edges, final_score, keys

def scoreNode(node, scores, keys, edges):
	"""
	Helper function to get the score of each node
	"""
	node_set = list(node)
	first = node_set[0]
	parts_gen = partition(node_set)
	unsorted_parts = [p for p in parts_gen]
	unsorted_parts = unsorted_parts[:-1]
	sorted_parts = []
	for p in unsorted_parts:
		sorted_p = []
		for i,v in enumerate(p):
			if first in list(v):
				sorted_p.append(p[i])
				break
		rest = [v for ind,v in enumerate(p) if ind is not i]
		sorted_p.extend(rest)
		sorted_parts.append(sorted_p)
	# only keep the partitions (groups of edges) that we have explored in the modeling process, store this as a dict so we can figure out which edge is best
	in_scores = dict()
	for part in sorted_parts:
		# put the members in the right format
		tups = [tuple(p) for p in part]
		in_scores[tuple(tups)] = 0
		# in_scores[tuple(tups)] += scoreTup(tups,scores,in_scores[tuple(tups)])
		in_scores[tuple(tups)] = scoreTup(tups,scores,in_scores[tuple(tups)])
	# once we have all the scores for that particular node, find the best score/path to that score and return
	# print node
	##### 
	max_edges = max(in_scores, key=in_scores.get)
	######
	key_inds = [keys.index(tuple(sorted(k))) for k in max_edges]
	# print max_edges
	# key_inds = [keys.index(k) for k in max_edges]
	max_score = in_scores[max_edges]
	return key_inds, max_score

def scoreTup(tup, scores, tup_score ):
	"""
	Helper function to get the score of a tuple
	"""
	# if we only have a single tuple
	if len(tup) == 1:
		tup_score += scores[tup[0]]
		return tup_score
	else:
	# elif len(tup) == 2:
	# get the likelihood of the dependent
		first = tup[0]
		tup_score = scoreTup([first],scores,tup_score)
		max_mod_all = []
		max_scr_all = []
		for independent in tup[1:]:
			max_mod_all = []
			max_scr_all = []
			# then get the best ways that each of the members of the dependent cluster merges with the independents	
			for t in first:
				pos_mod = genKeySeries((t,),sorted(list(independent)))
				pos_scrs = [scores[mod] for mod in pos_mod]
				#### this the standard
				max_scr = np.mean(pos_scrs)
				# max_scr_all.append(max_scr)
				####
			
				# max_scr = np.max(pos_scrs) # this is close
				# max_scr = np.min(pos_scrs)
				# max_scr = np.median(pos_scrs)
				# max_mod = pos_mod[pos_scrs.index(max_scr)]
				max_scr_all.append(max_scr)
				# max_mod_all.append(max_mod)
			
			#### this the standard
			tup_score += np.mean(max_scr_all) # this is close
			####
			# tup_score += np.min(max_scr_all)
			# tup_score += np.max(max_scr_all) 
		return tup_score

def saveTree(filename, tree, scores, final_score):
	"""
	Helper function to save the nodes
	"""
	with open(filename,'w') as f:
		for k in tree:
			f.write(str(k)+' ('+str(scores[k])+'): ')
			ch = []
			for v in tree[k]:
				ch.append(str(v))
			chstr = ', '.join(ch)
			f.write(chstr)
			f.write('\n')
		f.write('\n Total score: '+str(final_score))

def saveIndArr(filename,indarr,keys):
	"""
	Helper function to save the edges
	"""
	with open(filename,'w') as f:
		for ind in xrange(0,len(keys)):
			cur_key = keys[ind]
			children = indarr[ind]
			if children == -1:
				child_string = str(children)
			else:
				child_string = ', '.join(str(keys[c]) for c in children)
			f.write(str(cur_key)+': '+child_string+'\n')

def saveScores(filename, keys, scores):
	"""
	Helper function to save the scores
	"""
	with open(filename,'w') as f:
		for k in keys:
			f.write(str(k)+': '+str(scores[k])+'\n')
