import os

def mergeDicts(*dicts):
	"""
	Helper function to merge dictionaries together
	"""
	d = {}
	for di in dicts:
		for key in di:
			if key in d and di[key] < d[key]:
				pass
			else:
				d[key] = di[key]
	return d

def readNoiseGraph( filename ):
	g = dict()
	with open(filename,'r') as f:
		for line in f:
			if len(line.split()) == 0 or 'Total' in line: continue
			spl = line.strip().split(":")
			parent = tuple(sorted(spl[0].split(") (")[0].translate(None,"(,\'").split(" ")))
			children = spl[1].split("), (")
			children = [tuple(sorted(c.translate(None,"(,\')").strip().split(" "))) for c in children]
			g[parent] = sorted(children, key = len)
	return g

def getNoiseTreePaths( resdir ):
	net_files = os.listdir( resdir )
	noise_to_files = {}
	for f in net_files:
		if any(s in f for s in ['arr','scores','tmp']): continue
		n_val = f.split('_')[1]
		if n_val in noise_to_files:
			noise_to_files[n_val].append(resdir+f)
		else:
			noise_to_files[n_val] = [resdir+f]
	return noise_to_files

def getHoldTreePaths( resdir ):
	net_files = os.listdir( resdir )
	noise_to_files = {}
	for f in net_files:
		if any(s in f for s in ['arr','scores','tmp']): continue
		n_val = f.split('_tree')[0]
		if n_val in noise_to_files:
			noise_to_files[n_val].append(resdir+f)
		else:
			noise_to_files[n_val] = [resdir+f]
	return noise_to_files


def uniqueGraphs( gs ):
	ugs = []
	for g in gs:
		if g in ugs: continue
		ugs.append(g)
	return ugs

def countNets(nets, net):
	return len([cnet for cnet in nets if cnet == net])


def writeModelGraphMapping( noisedir , holddir , savedir ):

	n2f = getNoiseTreePaths(noisedir)
	h2f = getHoldTreePaths(holddir)

	n2f = mergeDicts(n2f,h2f)
	del n2f['.DS_Store']

	n2g = dict()
	n2gall = dict()

	for n in sorted(n2f.keys()):
		gr = []
		for f in n2f[n]:
			g = readNoiseGraph(f)
			gr.append(g)
		n2gall[n] = gr
		n2g[n] = uniqueGraphs(gr)

	augs = []
	for n in n2g.keys():
		for g in n2g[n]:
			if g not in augs:
				augs.append(g)

	g2n = dict()

	for n in n2g.keys():
		for g in n2g[n]:
			gi = augs.index(g)
			if gi not in g2n:
				g2n[gi] = [(n,countNets(n2gall[n],g))]
			else:
				g2n[gi].append((n,countNets(n2gall[n],g)))

	savefile = savedir+"/models_to_graphs.txt"

	with open(savefile, 'w') as f:
		f.write("index to network: \n")
		for gi in range(0,len(augs)):
			g = augs[gi]
			f.write(str(gi)+": \n")
			for n in g:
				toprint = " ".join(n)
				tp = ",".join([" ".join(no) for no in g[n]])
				f.write(toprint+":"+tp+"\n")
		f.write("\n")
		f.write("\n")
		f.write("network to model: \n")
		for g in g2n:
			f.write(str(g)+": ")
			for k in g2n[g]:
				f.write(k[0]+','+str(k[1])+"\n")


			