from getFilepaths import *
import os
import numpy as np
import itertools

genKeys = lambda rnas,order: list(itertools.permutations(rnas,order))

def mergeDicts(*dicts):
	d = {}
	for di in dicts:
		for key in di:
			if di[key] == 0:
				continue
			if key in d and di[key] < d[key]:
				pass
			else:
				d[key] = di[key]
	return d

def loadResFile(filepath, keylen):
	vals = []
	with open(filepath,'r') as f:
		for line in f:
			if keylen == 1:
				if line == '\n':
					break
				else:
					vals.append(float(line.translate(None, '[]').strip()))
	vmean = np.mean(vals)
	vvar = np.var(vals)
	return vals, vmean, vvar

def getPairModelRes(res_dir,keys):
	files = os.listdir(res_dir)
	likes = {}
	for k1 in keys:
		for k2 in keys:
			k = (k1,k2)
			for f in files:
				if f.startswith('_'.join(map(str,k))) and 'params' not in f and f.endswith('txt'):
					with open(res_dir+f,'r') as cf:
						cur_likes = []
						for line in cf.readlines():
							if line == '\n': 
								break
							elif ',' in line:
								line = line.split(',')[1]
							cur_likes.append(float(line.translate(None, '[]').strip()))
					likes[k] = cur_likes
	mean_likes = {}
	var_likes = {}
	for k in likes:
		if len(likes[k]) == 0:
			mean_likes[k] = 0
			var_likes[k] = 0
		else:
			mean_likes[k] = np.mean(likes[k])
			var_likes[k] = np.var(likes[k])
	return mean_likes, var_likes

def getModelResultsAll(rnas,keylen,resdir):
	keys = genKeys(rnas,keylen)
	files = os.listdir(resdir)
	means = {}
	varis = {}
	params = {}
	for k in keys:
		likes = []
		for f in files:
			if f.startswith('_'.join(map(str,k))) and f.endswith('.txt'):
				if f.endswith('params.txt'):
					continue
				else:
					with open(resdir+f,'r') as cf:			
						for line in cf.readlines():
							if ',' in line:
								line = line.split(',')[1]
							elif line == '\n': 
								break
							else:
								likes.append(float(line.translate(None, '[]').strip()))
						means[k] = np.mean(likes)
						varis[k] = np.var(likes)
			else: continue
	for key in means.keys():
		if str(means[key]) == 'nan':
			means.pop(key)
	return means, varis

def writePairModelRes(mlikes,vlikes,mfile,vfile,rnas):
	rnas = ['PB2','PB1','PA','HA','NP','NA','M','NS']
	with open(mfile,'w') as f:
		f.write(' ,'+','.join(rnas)+'\n')
		for rna in rnas:
			sorted_vals = ['%0.4f' % mlikes[(rna,rna2)] if rna2 != rna else ' ' for rna2 in rnas]
			outstr = ','.join(sorted_vals).replace('0.0000',' ')
			f.write(rna+','+outstr+'\n')
	with open(vfile,'w') as f:
		f.write(' ,'+','.join(rnas)+'\n')
		for rna in rnas:
			sorted_vals = ['%0.4f' % vlikes[(rna,rna2)] if rna2 != rna else ' ' for rna2 in rnas]
			outstr = ','.join(sorted_vals).replace('0.0000',' ')
			f.write(rna+','+outstr+'\n')			

def getCovarModelRes(res_dir, rnas):
	files = os.listdir(res_dir)
	mean_dict = {}
	var_dict = {}
	for rna in rnas:
		cur_file = [res_dir+f for f in files if f.startswith(rna) and f.endswith('.txt') and 'params' not in f]
		_, cur_mean, cur_var = loadResFile(cur_file[0],1)
		mean_dict[rna] = cur_mean
		var_dict[rna] = cur_var

	return mean_dict, var_dict

def writeCovarTable(cmeans, cvars, nmeans, nvars, cnmeans, cnvars, mean_file, var_file):
	keys = ['PB2','PB1','PA','HA','NP','NA','M','NS']
	with open(mean_file,'w') as f:
		f.write(' ,Cell Model,Nuclear Model,Cell+Nuclear Model\n')
		for k in keys:
			outstr = k+',%0.4f,%0.4f,%0.4f\n' % (-cmeans[k]/10.,-nmeans[k]/10.,-cnmeans[k]/10.)
			f.write(outstr)

	with open(var_file,'w') as f:
		f.write(' ,Cell Model,Nuclear Model,Cell+Nuclear Model\n')
		for k in keys:
			outstr = k+',%0.4f,%0.4f,%0.4f\n' % (cvars[k]/10.,nvars[k]/10.,cnvars[k]/10.)
			f.write(outstr)

def writeAllRes(filename,likes,varis):
	keys = ['PB2','PB1','PA','HA','NP','NA','M','NS']
	with open(filename,'w') as f:
		f.write('RNAs,CV Likelihood,CV Variance\n')
		for k in keys:
			key_order = [ks for ks in likes if ks[0]==k]
			for ks in key_order:
				f.write(' '.join(list(ks))+',%0.5f,%0.5f\n' % (-likes[ks],varis[ks]))

def writeMaxLikeTable(filename, rnas, likes, keylen):
	# rnas = ['PB2','PB1','PA','HA','NP','NA','M','NS']
	tab_file = filename
	keys = [k for k in likes if len(k) == keylen]
	max_vals = {}
	skeys = rnas

	for r in rnas:
		rvals = {k: -likes[k] for k in keys if k[0] == r}
		sorted_vals = sorted(rvals, key=rvals.get, reverse=True)
		mv = [sorted_vals[0]]
		for i in xrange(1,len(sorted_vals)):
			if sorted(list(mv[0])) == sorted(list(sorted_vals[i])):
				continue
			else:
				mv.append(sorted_vals[i])
				break
		max_vals[r] = mv 

	with open(tab_file,'w') as f:
		if keylen == 4 or keylen == 3:
			f.write('RNA,CV Likelihood,'+','.join(rnas)+'\n')
			for r in rnas:
				f.write(str(r)+'\n')
				for v in max_vals[r]:
					vs = [s for s in v]
					vs1 = [' '.join(vs)]
					vs1.append(str(likes[v]))
					pvs = {s: likes[tuple([r,s])] for s in vs if s is not r}
					pvsc = []
					for cr in skeys:
						if cr in pvs:
							pvsc.append(str(pvs[cr]))
						else:
							pvsc.append(' ')
					vs1.extend(pvsc)
					
					f.write(','.join(vs1)+'\n')
		else:
			for r in rnas:
				f.write(str(r)+'\n')
				for v in max_vals[r]:
					vs = [s for s in v]
					vs1 = [' '.join(vs)]
					
					vs1.append(str(likes[v]))
					f.write(','.join(vs1)+'\n')
		
def runWriteTables( cres , nres , cnres , pres , npres , tres , qres , table_figure_dir , finalresdir , rnas):
	rnas = ['PB2','PB1','PA','HA','NP','NA','M','NS']
	t2 = table_figure_dir+'/Table_2.csv'
	st2 = table_figure_dir+'/Supp_Table_2.csv'
	t3 = table_figure_dir+'/Table_3.csv'
	st3 = table_figure_dir+'/Supp_Table_3.csv'
	t4 = table_figure_dir+'/Table_4.csv'
	st4 = table_figure_dir+'/Supp_Table_4.csv'
	t5 = table_figure_dir+'/Table_5.csv'
	st5 = table_figure_dir+'/Supp_Table_5.csv'
	t6 = table_figure_dir+'/Table_6.csv'
	st6 = table_figure_dir+'/Supp_Table_6.csv'
	pdir = finalresdir+"/p_vals/"
	pfiles = [f for f in os.listdir(pdir) if f.endswith('.txt')]
	pdict = {}
	for f in pfiles:
		if 'DS' in f or '_' not in f: continue
		i = f.split('_')[1]
		pdict[i.split('.')[0]] = pdir+f

	for key in pdict.keys():
		if key == 'N':
			ent = pdict.pop(key)
			pdict['NA'] = ent

	pmeans = {}
	pvars = {}
	all_p = {}

	for rna in rnas:
		vals = []
		with open(pdict[rna],'r') as f:
			for line in f.readlines():
				if line.startswith('m') or line == '\n':
					break
				else:
					vals.append(float(line.strip()))
		pmeans[rna] = np.mean(vals)
		pvars[rna] = np.var(vals)
		all_p[rna] = vals

	with open(table_figure_dir+'Table_1.csv','w') as f:
		f.write(' ,Mean p-value,Variance\n')
		for key in rnas:
			outstr = ','.join([key,'%0.5f' %pmeans[key], '%0.5f' % pvars[key]])
			f.write(outstr+'\n')

	with open(table_figure_dir+'Supp_Table_1.csv','w') as f:
		f.write('RNA,P-value\n')
		for rna in rnas:
			for v in all_p[rna]:
				f.write(rna+',%0.5f\n' % v)



	writeCovarTable(cres[0],cres[1],nres[0],nres[1],cnres[0],cnres[1],t2,st2)

	plikes = pres[0]
	pvars = pres[1]
	plikes = {p:-plikes[p]/10.0 for p in plikes}
	writePairModelRes(plikes,pvars,t3,st3,rnas)

	cnplikes = npres[0]
	cnpvars = npres[1]
	cnplikes = {p:-cnplikes[p]/10.0 for p in cnplikes}
	writePairModelRes(cnplikes,cnpvars,t4,st4,rnas)

	cntmeans = tres[0]
	cntvars = tres[1]
	cntmeans = {p:-cntmeans[p]/10.0 for p in cntmeans}
	writeAllRes(st5,cntmeans,cntvars)

	cntmeans = mergeDicts(cntmeans,plikes,cnplikes)
	cntvars = mergeDicts(cntvars,pvars,cnpvars)
	writeMaxLikeTable(t5, rnas, cntmeans, 3)

	cnqmeans = qres[0]
	cnqvars = qres[1]
	cnqmeans = {p:cnqmeans[p]/10.0 for p in cnqmeans}

	writeAllRes(st6,cnqmeans,cnqvars)

	cnqmeans = mergeDicts(cnqmeans,cntmeans,plikes,cnplikes)
	cnqvars = mergeDicts(cnqvars,cntvars,pvars,cnpvars)
	writeMaxLikeTable(t6, rnas, cnqmeans, 4)